import java.awt.event.*;
import java.awt.*;
import java.text.*;
import java.util.*;

public class FontChooser extends java.applet.Applet 
                  implements ItemListener {

Choice         font;
Choice         size;
Checkbox       bold;
Checkbox       italic;
Label          preview;
Choice         color;
NumberFormat   format;
Locale         locale;
ResourceBundle resources;

public void init() {
  // Load the locale-specific resources
  // change your control panel settings to try out different locales

  locale = Locale.getDefault();

  // Or, you can write statements like this:
  //   locale = new Locale("de","DE");
  //   locale = new Locale("es","ES");

  resources = ResourceBundle.getBundle("MyStrings", locale);

  // Create all the controls we need

  // The list of font face names
  font = new Choice();
  String[] availFonts = sort(Toolkit.getDefaultToolkit().getFontList());
  for (int i = 0; i < availFonts.length; i++)
	font.addItem(availFonts[i] );
  font.select(0);
  font.addItemListener(this);

  // The point size
  // Use the number formatter to create a list of point sizes
  size = new Choice();
  format = NumberFormat.getInstance(locale);
  for (int i=8; i<37; i=i+2)
    size.addItem(format.format(i));
  size.select(size.getItemCount()-1);
  size.addItemListener(this);

  // The style
  bold = new Checkbox(resources.getString("Bold"));
  bold.addItemListener(this);
  italic = new Checkbox(resources.getString("Italic"));
  italic.addItemListener(this);

  // The color
  color = new Choice();
  color.addItem(resources.getString("Red"));
  color.addItem(resources.getString("Blue"));
  color.addItem(resources.getString("Green"));
  color.select(0);
  color.addItemListener(this);

  // A preview area
  preview = new Label(resources.getString("Sample"),Label.CENTER);

  // Now create the UI using a Border and Grid Layouts
  // The preview text fills the east side of the applet
  // On the west side, the textfields are arranged in a simple grid

  setLayout(new BorderLayout(5,5));
  Panel fields = new Panel();


  // Create a grid layout of 5 rows and 2 columns
  // Use a horizontal gap of 5 and vertical gap of 0
  fields.setLayout(new GridLayout (5,2,5,0));  
  fields.add (new Label(resources.getString("FontName"),Label.RIGHT));
  fields.add (font);
  fields.add (new Label(resources.getString("Size"),Label.RIGHT));
  fields.add (size);
  fields.add (new Label()); // just an empty placeholder
  fields.add (bold);
  fields.add (new Label()); // just an empty placeholder
  fields.add (italic);
  fields.add (new Label(resources.getString("Color"),Label.RIGHT));
  fields.add (color);
  add (BorderLayout.WEST, fields);
  add (BorderLayout.EAST, preview);
  setFont();
  doLayout();
 }

public void start() {
 }

public void stop() {
 }


//
// this method is called whenever anything is selected 
// in the choices or checkboxes
//
public void itemStateChanged(ItemEvent e) {
   setFont();
 }


//
// Calculate the new Font object and update the preview
// text with the right font and color
//
public void setFont(){
  Number   pointSize;
  String   selectedColor;
  Color    c = Color.red;
  int      style = Font.PLAIN;

  // set the new font
  if ( bold.getState() )
	style += Font.BOLD;
  if ( italic.getState() )
	style += Font.ITALIC;

  format.setParseIntegerOnly(true);
  try {
  	pointSize = format.parse(size.getSelectedItem());
  }
  catch (Exception e) {
    System.out.println(e);
    pointSize = (Number)new Integer(36);
  }
  preview.setFont(new Font(font.getSelectedItem(),
  						   style,
						   pointSize.intValue()));

  // set the color
  selectedColor = color.getSelectedItem();
  if (selectedColor.equals(resources.getString("Red")))
	c = Color.red;
  else if (selectedColor.equals(resources.getString("Blue")))
	c = Color.blue;
  else if (selectedColor.equals(resources.getString("Green")))
	c = Color.green;
  preview.setForeground(c);
 }


//
// sort the list of font names
// using the Collator class
//
public String[] sort (String[] unsorted){
  String temp;
  Collator collator = Collator.getInstance(locale);
  collator.setStrength(Collator.PRIMARY);
  for (int i=0; i<unsorted.length; i++) {
    for (int j=i; j<unsorted.length; j++) {
      if( collator.compare(unsorted[i],unsorted[j]) > 0 ) {
	    temp = unsorted[j];
		unsorted[j] = unsorted[i];
		unsorted[i] = temp;
	    }    
    }
  }
  return unsorted;
 }

}
