/*

DataContainingMessage.java

*/

package cs141b.gndo;

import info.net.Place;
import java.util.Vector;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;

public class DataContainingMessage extends VarMessage {
  private byte[] _data;
  private long _stateID;
  private String _dataType;

  public DataContainingMessage() {
    super();
    _data = null;
    _stateID = -1;
    _dataType = null;
  }

  public DataContainingMessage(Place from, Path where) {
    super(from, where);
    
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    DataOutputStream dos = new DataOutputStream(baos);

    try {
      where.leaf().writeData(dos);
      _stateID = where.leaf().currentState();
    }
    catch(IOException e) {
      Assert.error("IOException: "+e.toString());
    }

    _data = baos.toByteArray();
    Assert.isNotNull(_data);
    Assert.isTrue(_data.length >= 0);

    _dataType = where.leaf().getClass().getName();
  }

  public long stateID() {
    return _stateID;
  }

  public void readData(DataInputStream dis) throws IOException {
    super.readData(dis);

    _stateID = dis.readLong();

    int len = dis.readInt();
    Assert.isTrue(len >= 0);
    _data = new byte[len];
    dis.read(_data, 0, len);

    len = dis.readInt();
    StringBuffer dataTypeBuffer = new StringBuffer(len);
    for(int i=0;i<len;++i) {
      dataTypeBuffer.append(dis.readChar());
    }
    _dataType = dataTypeBuffer.toString();
  }

  public void writeData(DataOutputStream dos) throws IOException {
    super.writeData(dos);

    dos.writeLong(_stateID);

    Assert.isNotNull(_data);
    Assert.isTrue(_data.length >= 0);
    dos.writeInt(_data.length);
    dos.write(_data, 0, _data.length);

    Assert.isNotNull(_dataType);

    dos.writeInt(_dataType.length());
    dos.writeChars(_dataType);
  }

  public void update(SharedVar var) {
    Assert.isNotNull(_data);
    Assert.isTrue(_data.length >= 0);
    Assert.isTrue(var.currentState() < _stateID);

    Assert.isTrue(_dataType.equals(var.getClass().getName()));


    ByteArrayInputStream bais = new ByteArrayInputStream(_data);
    DataInputStream dis = new DataInputStream(bais);

    try {
      var._readDataFrom = from();

      var.readData(dis);
    }
    catch(IOException e) {
      Assert.error("Unable to read"+e.toString());
    }

    var.setBestGuessOwner(from());
    var.setCurrentState(_stateID);
  }
};
