/*

NewOwnerMessage.java - The message from a node sent to the person who's leaving,
asking for who we should reassign our best guesses to for a list of fields

*/

package cs141b.gndo;

import info.net.Place;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;

public class NewOwnerMessage extends Message {
  private PathList _whichFieldsGaveLock;
  private Vector _newGaveLock;
  private PathList _whichFieldsBestGuesses;
  private Vector _newBestGuesses;

  public NewOwnerMessage() {
    super();
  }

  public NewOwnerMessage(Place from, 
                         PathList whichFieldsGaveLock,
                         Vector newGaveLock,
                         PathList whichFieldsBestGuesses,
                         Vector newBestGuesses) {
    super(from);
    _whichFieldsGaveLock = whichFieldsGaveLock;
    _newGaveLock = newGaveLock;
    _whichFieldsBestGuesses = whichFieldsBestGuesses;
    _newBestGuesses = newBestGuesses;
  }

  public PathList whichFieldsGaveLock() {
    return _whichFieldsGaveLock;
  }

  public Vector newGaveLock() {
    return _newGaveLock;
  }

  public PathList whichFieldsBestGuesses() {
    return _whichFieldsBestGuesses;
  }

  public Vector newBestGuesses() {
    return _newBestGuesses;
  }

  public void readData(DataInputStream dis) throws IOException {
    super.readData(dis);
    int len = dis.readInt();
    _whichFieldsGaveLock = new PathList();
    for(int i=0;i<len;++i) {
      _whichFieldsGaveLock.add((Path)readObject(dis));
    }
    _newGaveLock = new PathList();
    for(int i=0;i<len;++i) {
      boolean haveItem = dis.readBoolean();
      if (haveItem) {
        _newGaveLock.addElement((Place)readObject(dis));
      } else {
        _newGaveLock.addElement(null);
      }
    }
    len = dis.readInt();
    _whichFieldsBestGuesses = new PathList();
    for(int i=0;i<len;++i) {
      _whichFieldsBestGuesses.add((Path)readObject(dis));
    }
    _newBestGuesses = new PathList();
    for(int i=0;i<len;++i) {
      boolean haveItem = dis.readBoolean();
      if (haveItem) {
        _newBestGuesses.addElement((Place)readObject(dis));
      } else {
        _newBestGuesses.addElement(null);
      }
    }
  }

  public void writeData(DataOutputStream dos) throws IOException {
    super.writeData(dos);
    dos.writeInt(_whichFieldsGaveLock.size());
    for(int i=0;i<_whichFieldsGaveLock.size();++i) {
      ((Path)_whichFieldsGaveLock.elementAt(i)).writeObject(dos);
    }
    for(int i=0;i<_newGaveLock.size();++i) {
      Place curGaveLock = (Place)_newGaveLock.elementAt(i);
      if (curGaveLock == null) {
        dos.writeBoolean(false);
      } else {
        dos.writeBoolean(true);
        curGaveLock.writeObject(dos);
      }
    }
    dos.writeInt(_whichFieldsBestGuesses.size());
    for(int i=0;i<_whichFieldsBestGuesses.size();++i) {
      ((Path)_whichFieldsBestGuesses.elementAt(i)).writeObject(dos);
    }
    for(int i=0;i<_newBestGuesses.size();++i) {
      Place curBestGuess = (Place)_newBestGuesses.elementAt(i);
      if (curBestGuess == null) {
        dos.writeBoolean(false);
      } else {
        dos.writeBoolean(true);
        curBestGuess.writeObject(dos);
      }
    }
  }
};
