/*

Path.java

*/

package cs141b.gndo;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;

public class Path extends Message {
  static final int NOT_MADE_YET = 0;
  static final int FIELD_DELETED = 1;

  // Member variables 

  private Vector _containerList;
  private SharedVar _leaf;
  private int[] _indicies;
  private String _string;
  private boolean _valid;

  // Override functions

  public Path() {
    _containerList = null;
    _leaf = null;
    _indicies = null;
    _string = null;
    _valid = false;
  }

  public void writeData(DataOutputStream dos) throws IOException {
    Assert.isTrue(_valid);
    Assert.isNotNull(_indicies);

    dos.writeInt(_indicies.length);
    for(int i=0;i<_indicies.length;++i) {
      dos.writeInt(_indicies[i]);
    }
  }

  public void readData(DataInputStream dis) throws IOException {
    int size = dis.readInt();
    _indicies = new int[size];
    for(int i=0;i<size;++i) {
      _indicies[i] = dis.readInt();
    }
    _string = null;
    _valid = false;
  }

  // Package functions

  Path(SharedVar leaf) {
    _leaf = leaf;
    _containerList = new Vector();

    SharedVar cur = _leaf._parent;
    while(cur != null) {
      _containerList.insertElementAt(cur, 0);
      cur = cur._parent;
    }
    _indicies = new int[_containerList.size()];
    for(int i=1;i<_containerList.size();++i) {
      cur = (SharedVar)_containerList.elementAt(i);

      _indicies[i-1] = cur._myIndex;
    }
    if (_containerList.size()>0) {
      Assert.isTrue(leaf._myIndex >= 0);
      _indicies[_containerList.size()-1] = _leaf._myIndex;
    }
    _string = null;
    _valid = true;
  }

  final void update(NDO ndo) {
    _valid = true;
    _containerList = new Vector();

    SharedVar cur = ndo;
    int i=0;

    while(i<_indicies.length && _valid) {
      _containerList.addElement(cur);

      try {
        cur = ((SharedContainer)cur).field(_indicies[i]);
        Assert.isNotNull(cur);
        ++i;
      }
      catch(FieldDeletedException e) {
        _valid = false;
	  }
      catch(ArrayIndexOutOfBoundsException e) {
        _valid = false;
      }
    }

    _leaf = cur;
  }

  final SharedContainer container(int index) {
    Assert.isTrue(_valid);

    return (SharedContainer)_containerList.elementAt(index);
  }

  final SharedContainer getLastValid() {
    Assert.isFalse(_valid);

    return (SharedContainer)_containerList.elementAt(_containerList.size()-1);
  }

  final int whyInvalid() {
    Assert.isNotNull(_indicies);
    Assert.isFalse(_valid);

    if (getLastValid().hasFieldAlreadyExisted(_indicies[_containerList.size()-1])) {
      return FIELD_DELETED;
    } else {
      return NOT_MADE_YET;
    }
  }

  final SharedVar leaf() {
    Assert.isTrue(_valid);

    return _leaf;
  }

  boolean valid() {
    return _valid;
  }

  public boolean equals(Object obj) {
    Assert.isNotNull(_indicies);

    if (obj == null) return false;
    
    Path other = (Path)obj;

    Assert.isNotNull(other._indicies);

    if (_indicies.length != other._indicies.length) return false;

    for(int i=0;i<_indicies.length;++i) {
      if (_indicies[i] != other._indicies[i]) return false;
    }

    return true;
  }

  boolean lessThan(Path other) {
    Assert.isNotNull(_indicies);
    Assert.isNotNull(other._indicies);

    int maxIndex = _indicies.length;
    if (other._indicies.length > maxIndex) maxIndex = other._indicies.length;

    for(int i=0;i<maxIndex;++i) {
      if (_indicies[i] < other._indicies[i]) return true;
      if (_indicies[i] > other._indicies[i]) return false;
    }

    if (other._indicies.length > maxIndex) return true;

    return false;
  }

  int compare(Path other) {
    Assert.isNotNull(_indicies);
    Assert.isNotNull(other._indicies);

    int minIndex = _indicies.length;
    if (other._indicies.length < minIndex) minIndex = other._indicies.length;

    for(int i=0;i<minIndex;++i) {
      if (_indicies[i] < other._indicies[i]) return -1;
      if (_indicies[i] > other._indicies[i]) return 1;
    }

    if (other._indicies.length > minIndex) return -1;
    if (_indicies.length > minIndex) return 1;

    return 0;
  }
  
  public String toString() {
    if (_string == null) {
      StringBuffer outBuf = new StringBuffer("[");

      if (_containerList != null) {
        for(int i=1;i<_containerList.size();++i) {
          SharedContainer cur = (SharedContainer)_containerList.elementAt(i);
          
          outBuf.append(cur._myIndex);
          outBuf.append(" ");
        }
        outBuf.append(_leaf._myIndex);
      } else if (_indicies != null) {
        for(int i=0;i<_indicies.length;++i) {
          if (i>0) outBuf.append(" ");
          outBuf.append(_indicies[i]);
        }
      } else {
        outBuf.append("Invalid");
      }

      outBuf.append("]");

      _string = outBuf.toString();
    }
    
    Assert.isNotNull(_string);
    return _string;
  }
};
