/*

SharedContainer.java

*/

package cs141b.gndo;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import info.net.Place;

public class SharedContainer extends SharedVar {
  private Hashtable _childList;
  private Vector _typeList;
  private int _nextID;

  SharedContainer() {
    super();
    _childList = new Hashtable();
    _typeList = new Vector();
  }

  /**The constructor - Same as superclass
    @see cs141b.gndo.SharedVar#SharedVar*/
  public SharedContainer(SharedContainer parent) {
    super(parent);
    _childList = new Hashtable();
    _typeList = new Vector();
  }

  /** get a reference to a field by its ID number */
  public synchronized SharedVar field(int n) throws FieldDeletedException {
    Assert.isTrue(n>=0);
    VarAndType vat = (VarAndType)_childList.get(new Integer(n));

    if (vat == null) {
      if (hasFieldAlreadyExisted(n)) {
        throw new FieldDeletedException();
      } else {
        throw new ArrayIndexOutOfBoundsException(n);
      }
    }

    Assert.isNotNull(vat.var());

    return vat.var();
  }

  /** find out what the maximum field ID is */
  public synchronized int nextID() {
    return _nextID;
  }

  public synchronized boolean hasFieldAlreadyExisted(int n) {
    Assert.isTrue(n>=0);
    return n < _nextID;
  }
  /** Check for the existence of a field */
  public synchronized boolean doesFieldExist(int n) {
    Assert.isTrue(n>=0);
    if (null==_childList.get(new Integer(n))) {
      return false;
    } else {
      return true;
    }

  }

  /** remove a field - Note: both this container and the child field must be locked. */
  public synchronized void removeField(int n) {
    SharedVar child = null;
    try {
      child = field(n);
    }
    catch (FieldDeletedException e) {
      throw new FieldNotLockedException("SharedContainer must be locked to remove a field.");
    }
    if (!locked()) { 
      throw new FieldNotLockedException("SharedContainer must be locked to remove a field.");
    }
    if (!child.locked()) { 
      throw new FieldNotLockedException("Child SharedVar must be locked to remove a field.");
    }
    
    child.delete();
    _childList.remove(new Integer(n));

    setModified();
  }

  // Package functions

  synchronized void addField(SharedVar var) {
    if (!locked()) { 
      throw new FieldNotLockedException("SharedContainer not locked.");
    }

    // Check type database

    String thisTypeName = var.getClass().getName();
    boolean done = false;
    int i=0;
    while(i<_typeList.size() && !done) {
      if (thisTypeName.equals(_typeList.elementAt(i))) {
        done = true;
      } else {
        ++i;
      }
    }
    if (!done) {
      _typeList.addElement(thisTypeName);
    }
    Assert.isTrue(thisTypeName.equals(_typeList.elementAt(i)));

    var._myIndex = _nextID;
    _childList.put(new Integer(_nextID), new VarAndType(var, i));
    ++_nextID;

    setModified();
  }

  synchronized void initialSetupPropogate(Place where) {
    initialSetupPropogateChildren(where);

    super.initialSetupPropogate(where);
  }

  synchronized void initialSetupPropogateChildren(Place where) {
    java.util.Enumeration children = _childList.elements();
    while(children.hasMoreElements()) {
      SharedVar curChild = ((VarAndType)children.nextElement()).var();

      curChild.initialSetupPropogate(where);
    }
  }

  synchronized void delete() {
    super.delete();
    
    java.util.Enumeration children = _childList.elements();
    while(children.hasMoreElements()) {
      SharedVar curChild = ((VarAndType)children.nextElement()).var();

      curChild.delete();
    }
    
  }

  synchronized void pushObject(Place where) {
    // Push ourselves, if needed.
    super.pushObject(where);

    pushChildren(where);
  }

  synchronized void pushChildren(Place where) {
    // Push our children
    java.util.Enumeration children = _childList.elements();
    while(children.hasMoreElements()) {
      SharedVar curChild = ((VarAndType)children.nextElement()).var();
      curChild.pushObject(where);
    }
  }

  synchronized void findPointersTo(PathList resultBinGaveLock, PathList resultBinBestGuesses,
                                   Place findLocation) {
    super.findPointersTo(resultBinGaveLock, resultBinBestGuesses, 
                         findLocation);

    java.util.Enumeration children = _childList.elements();
    while(children.hasMoreElements()) {
      SharedVar curChild = ((VarAndType)children.nextElement()).var();
      curChild.findPointersTo(resultBinGaveLock, resultBinBestGuesses, 
                              findLocation);
    }
  }

  public synchronized void lockAll() {
    lock();
    
    Enumeration children = _childList.elements();
    while(children.hasMoreElements()) {
      SharedVar curChild = ((VarAndType)children.nextElement()).var();
      curChild.lockAll();
    }
  }

  public synchronized void unlockAll() {
    if (locked()) unlock();
    
    java.util.Enumeration children = _childList.elements();
    while(children.hasMoreElements()) {
      SharedVar curChild = ((VarAndType)children.nextElement()).var();

      curChild.unlockAll();
    }
  }

  public synchronized void readData(DataInputStream dis) throws IOException {
    int newTypeListSize = dis.readInt();
    for(int i=0;i<newTypeListSize;++i) {
      int curTypeLen = dis.readInt();
      StringBuffer curTypeBuf = new StringBuffer(curTypeLen);

      for(int j=0;j<curTypeLen;++j) {
        curTypeBuf.append(dis.readChar());
      }

      String curType = curTypeBuf.toString();

      Assert.isTrue(i <= _typeList.size());
      Assert.isTrue(i == _typeList.size() || curType.equals(_typeList.elementAt(i)));
      if (i == _typeList.size()) {
        _typeList.addElement(curType);
      }
    }

    _nextID = dis.readInt();
    int newChildListLength = dis.readInt();
    Hashtable newChildList = new Hashtable();

    try {
      for(int i=0;i<newChildListLength;++i) {
        Integer newIndex = new Integer(dis.readInt());
        int newType = dis.readInt();
        
        VarAndType vat = (VarAndType)_childList.get(newIndex);
        if (vat == null) {
          // This is a new item
          
          String newFieldType = (String)_typeList.elementAt(newType);
          
          Assert.isTrue(!newChildList.containsKey(newIndex));
          
          SharedVar newClass = (SharedVar)Class.forName(newFieldType).newInstance();
          newClass._parent = this;
          if (_parent == null) {
            newClass._ndo = (NDO)this;
          } else {
            Assert.isNotNull(_ndo);
            newClass._ndo = _ndo;
          }
          newClass._myIndex = newIndex.intValue();
          newChildList.put(newIndex, new VarAndType(newClass, newType));

          newClass.initialSetupPropogate(_readDataFrom);
        } else {
          // This is an old item, just move into new list
          
          newChildList.put(newIndex, vat);
          _childList.remove(newIndex);
        }
      }
    }
    catch(ClassNotFoundException e) {
      Assert.error("ClassNotFoundException: "+e);
    }
    catch(InstantiationException e) {
      Assert.error("InstantiationException: "+e);
    }
    catch(IllegalAccessException e) {
      Assert.error("IllegalAccessException: "+e);
    }

    Enumeration e = _childList.elements();
    while(e.hasMoreElements()) {
      VarAndType vat = (VarAndType)e.nextElement();
      Assert.isNotNull(vat);

      vat.var().delete();
    }

    _childList = newChildList;
/*
System.out.println("_nextID=="+_nextID);
System.out.println("_typeList.size()=="+_typeList.size());
System.out.print("_typeList == [");
for(int i=0;i<_typeList.size();++i) {
String cur = (String)_typeList.elementAt(i);
if (i>0) System.out.print(" ");
System.out.print(cur);
}
System.out.println("]");
System.out.println("_childList.size()=="+_childList.size());
System.out.println("[[[[[[[[[[[[[[[[");
Enumeration e = _childList.keys();
while(e.hasMoreElements()) {
Integer curIndex = (Integer)e.nextElement();
VarAndType vat = (VarAndType)_childList.get(curIndex);
System.out.println(curIndex+" type=="+vat.type()+"  "+vat.var().getClass().getName());
}
System.out.println("]]]]]]]]]]]]]]]]");
*/
  }

  public synchronized void writeData(DataOutputStream dos) throws IOException {
    dos.writeInt(_typeList.size());
    for(int i=0;i<_typeList.size();++i) {
      String curType = (String)_typeList.elementAt(i);

      dos.writeInt(curType.length());
      dos.writeChars(curType);
    }
    dos.writeInt(_nextID);
    dos.writeInt(_childList.size());
    Enumeration e = _childList.keys();
    while(e.hasMoreElements()) {
      Integer index = (Integer)e.nextElement();
      VarAndType vat = (VarAndType)_childList.get(index);

      Assert.isNotNull(vat);

      dos.writeInt(index.intValue());
      dos.writeInt(vat.type());
    }
  }
};
