//----------------------------------------------------------------------
// Shared Vector
// by Louis Thomas
// Last modified: 6-2-97

package cs141b.gndo;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;

//----------------------------------------------------------------------
// The shared vector class
public class SharedVector extends SharedContainer {

  //------------------------------------------------------------------
  // Member Variables
  Vector m_vAssocList;

  //------------------------------------------------------------------
  // Constructor
  public SharedVector(SharedContainer scParent) {
    super(scParent);
    m_vAssocList=new Vector();
  }

  //------------------------------------------------------------------
  // Constructor
  // Only the NDO is its own parent
  public SharedVector() {
    super();
    m_vAssocList=new Vector();
  }

  //------------------------------------------------------------------
  // override removeField() so that it cleans up after itself
  public synchronized void removeField(int n) {
    m_vAssocList.removeElement(new Integer(n));
    super.removeField(n);
  }

  //------------------------------------------------------------------
  // return a field
  public synchronized SharedVar elementAt(int nID) {
    Assert.isTrue(nID>=0 && nID<m_vAssocList.size());
    Integer iTemp=(Integer)m_vAssocList.elementAt(nID);
    try {
      return super.field(iTemp.intValue());
    }
    catch(FieldDeletedException e) {
      Assert.error("Field deleted in container, but not in vector");
    }
    return null;
  }

  //------------------------------------------------------------------
  // remove a field
  public synchronized void removeElementAt(int nID) {
    Assert.isTrue(nID>=0 && nID<m_vAssocList.size());
    if (!locked()) {
      throw new FieldNotLockedException("SharedVector not locked.");
    }
    Integer iTemp=(Integer)m_vAssocList.elementAt(nID);
    m_vAssocList.removeElementAt(nID);
    setModified();
    super.removeField(iTemp.intValue());
  }

  //------------------------------------------------------------------
  // move a field within a vector
  public synchronized void moveElementAt(int nID, int nDelta){
    Assert.isTrue(nID>=0 && nID<m_vAssocList.size());
    Assert.isTrue(nDelta!=0);
    Assert.isTrue(nID+nDelta>=0 && nID+nDelta<m_vAssocList.size());
    if (!locked()) { 
      throw new FieldNotLockedException("SharedVector not locked.");
    }
    Integer iTemp=(Integer)m_vAssocList.elementAt(nID);
    m_vAssocList.removeElementAt(nID);
    m_vAssocList.insertElementAt(iTemp, nID+nDelta);
    setModified();
  }

  //------------------------------------------------------------------
  // move the last element to a given position
  public synchronized void moveLastElementTo(int nPos) {
    Assert.isTrue(nPos>=0 && nPos<m_vAssocList.size());
    if (!locked()) { 
      throw new FieldNotLockedException("SharedVector not locked.");
    }
    if (nPos != m_vAssocList.size()-1) {
      Integer iTemp=(Integer)m_vAssocList.elementAt(m_vAssocList.size()-1);
      m_vAssocList.removeElementAt(m_vAssocList.size()-1);
      m_vAssocList.insertElementAt(iTemp, nPos);
      setModified();
    }
  }

  //------------------------------------------------------------------
  // return number of fields in vector
  public synchronized int size() {
    return m_vAssocList.size();
  }

  //------------------------------------------------------------------
  // Add a field to the end of the vector
  synchronized void addField(SharedVar var) {
    if (!locked()) { 
      throw new FieldNotLockedException("SharedVector not locked.");
    }
    super.addField(var);
    m_vAssocList.addElement(new Integer(var._myIndex));
  }

  //------------------------------------------------------------------
  /** Read in all of the fields not derived from SharedVar from an outside stream */
  public synchronized void readData(DataInputStream dis) throws IOException {
    super.readData(dis);
    int nSize=dis.readInt();
    m_vAssocList=new Vector(nSize);
    for (int nIndex=0; nIndex<nSize; nIndex++) {
      m_vAssocList.addElement(new Integer(dis.readInt()));
    }
  }

  //------------------------------------------------------------------
  /** Write out all of the fields not derived from SharedVar to an outside stream */
  public synchronized void writeData(DataOutputStream dos) throws IOException {
    super.writeData(dos);
    dos.writeInt(m_vAssocList.size());
    for (int nIndex=0; nIndex<m_vAssocList.size(); nIndex++) {
      dos.writeInt(((Integer)m_vAssocList.elementAt(nIndex)).intValue());
    }
  }

}
