/*

StateInt.java - A state variable that can be synchronized

*/

package cs141b.gndo;

class StateInt {
  //Private member variables

  private int _value;
  private boolean _locked;

  //Public member functions

  /**Constructor, requires initial value*/
  public StateInt(int value) {
    _value = value;
    _locked = false;
  }

  /**Blocks until this thread acquires the lock on this object*/
  public synchronized void lock() {
    try {
      while(_locked) {
        wait();
      }
      _locked = true;
    }
    catch(InterruptedException e) {
      Assert.error("Interrupted exception thrown: "+e.toString());
    }
  }

  /**Unlocks this thread's lock on this object*/
  public synchronized void unlock() {
    Assert.isTrue(_locked);

    _locked = false;
    notify();
  }

  /**Returns the value of the integer*/
  public synchronized int value() {
    Assert.isTrue(_locked);
    
    return _value;
  }

  /**Returns the value of the integer in a non-guarenteed way*/
  public int quickValue() {
    return _value;
  }

  /**Sets the value of the integer*/
  public synchronized void setValue(int value) {
    Assert.isTrue(_locked);

    _value = value;
  }

  /**Blocks until value is changed*/
  public synchronized void waitForChange() {
    Assert.isTrue(_locked);

    _locked = false; //Unlock object so others can lock and modify it
    notify();

    int curValue = _value;
    try {
      while(curValue == _value || _locked) {
        wait();
      }
    }
    catch(InterruptedException e) {
      Assert.error("Interrupted exception thrown: "+e.toString());
    }

    _locked = true; //Lock object so we return in the same state we entered
  }

  /**Blocks until value becomes the specified one*/
  public synchronized void waitForValue(int value) {
    Assert.isTrue(_locked);

    _locked = false; //Unlock object so others can lock and modify it
    notify();

    try {
      while(_value != value || _locked) {
        wait();
      }
    }
    catch(InterruptedException e) {
      Assert.error("Interrupted exception thrown: "+e.toString());
    }

    _locked = true; //Lock object so we return in the same state we entered
  }

  /**Blocks until value becomes other than the specified one*/
  public synchronized void waitForNotValue(int value) {
    Assert.isTrue(_locked);

    _locked = false; //Unlock object so others can lock and modify it
    notify();

    try {
      while(_value == value || _locked) {
        wait();
      }
    }
    catch(InterruptedException e) {
      Assert.error("Interrupted exception thrown: "+e.toString());
    }

    _locked = true; //Lock object so we return in the same state we entered
  }
};
