//----------------------------------------------------------------------
// SEdit
// by Louis Thomas
// Last modified: 6-03-97

package cs141b.sedit2;

import cs141b.gndo.SharedString;
import cs141b.gndo.SharedContainer;
import cs141b.gndo.SharedVector;
import cs141b.gndo.FieldDeletedException;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.MenuBar;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Panel;
import java.awt.Scrollbar;
import java.awt.TextField;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Observer;
import java.util.Observable;

//----------------------------------------------------------------------
// The Simultaneous Editor class
public class SEdit extends Frame implements Observer{

	//------------------------------------------------------------------
	// Member Variables
	private static final String mc_szAppName="SimulEdit";

	private SEditNDO m_seNDO;

	private int m_nTextVOffset;
	private int m_nTextHOffset;

	private Color m_cCursFg;
	private Color m_cCursBg;
/*
	private boolean m_bCursVisible;
	private int m_nCursLastXScreen;
	private int m_nCursLastYScreen;
	private int m_nCursLastXFile;
	private int m_nCursLastYFile;
	private char m_chCursLast;
*/
	private StatusBar m_statusbar;
	private Panel m_pnlCenter;
	private TextGrid m_textgrid;
	private Scrollbar m_sbVert;
	private Scrollbar m_sbHoriz;
	private Canvas m_canvEmpty;

	private MenuItem m_miNew;
	private MenuItem m_miOpen;
	private MenuItem m_miJoin;
	private MenuItem m_miClose;
	private MenuItem m_miSaveAs;
	private MenuItem m_miQuit;
	private MenuItem m_miAbout;
	private MenuItem m_miCursorColor;
	private MenuItem m_miTabSize;
	private MenuItem m_miJoinAddr;
	private MenuItem m_miCut;
	private MenuItem m_miCopy;
	private MenuItem m_miPaste;


	//------------------------------------------------------------------
	// Main - program execution starts here
	public static void main(String rgszArgs[]) {
		try {
		System.out.println("Starting app");
		SEdit win=new SEdit();
		System.out.println("quitting main");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	//------------------------------------------------------------------
	// Constructor
	public SEdit() {
		super(mc_szAppName);

		createMenus();
		setMenusToNoFile();
		setLayout(new BorderLayout());

		// create the status bar
		m_statusbar=new StatusBar();
		add("South", m_statusbar);
		m_statusbar.setMessage("Welcome to SimulEdit");

		m_canvEmpty=new Canvas();
		m_canvEmpty.setBackground(Color.gray);
		add("Center", m_canvEmpty);

		setBackground(Color.lightGray);
		resize(300,300);
		//setResizable(false);
		show();

		m_cCursFg=Color.white;
		m_cCursBg=Color.darkGray;

	}

	//------------------------------------------------------------------
	// Create the center panel with textgrid and scrollbars
	private void createCenterPanel() {

		m_pnlCenter=new Panel();
		m_sbVert=new Scrollbar(Scrollbar.VERTICAL);
		m_sbHoriz=new Scrollbar(Scrollbar.HORIZONTAL);
		m_textgrid=new TextGrid(40, 20, new Font("Courier", Font.PLAIN, 12), Color.black, Color.white);

		m_pnlCenter.setBackground(Color.gray);
		m_sbVert.setBackground(Color.lightGray);
		m_sbHoriz.setBackground(Color.lightGray);

		GridBagLayout gbl=new GridBagLayout();
		GridBagConstraints gbc=new GridBagConstraints();
		m_pnlCenter.setLayout(gbl);

		gbc.anchor=GridBagConstraints.NORTHWEST;
		gbc.gridx=0;
		gbc.gridy=0;
		gbl.setConstraints(m_textgrid, gbc);
		m_pnlCenter.add(m_textgrid);

		gbc.gridx=1;
		gbc.gridy=0;
		gbc.fill=GridBagConstraints.VERTICAL;
		gbl.setConstraints(m_sbVert, gbc);
		m_pnlCenter.add(m_sbVert);

		gbc.gridx=0;
		gbc.gridy=1;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbl.setConstraints(m_sbHoriz, gbc);
		m_pnlCenter.add(m_sbHoriz);

		remove(m_canvEmpty);
		add("Center", m_pnlCenter);
		pack();
	}

	//------------------------------------------------------------------
	// Create all the menus
	private void createMenus() {
		MenuBar mbar=new MenuBar();
		Menu mFile=new Menu("File");
		Menu mHelp=new Menu("Help");
		Menu mEdit=new Menu("Edit");
		Menu mSettings=new Menu("Settings");

		mFile.add(m_miNew=new MenuItem("New"));
		mFile.add(m_miOpen=new MenuItem("Open..."));
		mFile.add(m_miJoin=new MenuItem("Join..."));
		mFile.add(m_miClose=new MenuItem("Close"));
		mFile.addSeparator();
		mFile.add(m_miSaveAs=new MenuItem("Save As..."));
		mFile.addSeparator();
		mFile.add(m_miQuit=new MenuItem("Quit"));

		mHelp.add(m_miAbout=new MenuItem("About"));

		mEdit.add(m_miCut=new MenuItem("Cut"));
		mEdit.add(m_miCopy=new MenuItem("Copy"));
		mEdit.add(m_miPaste=new MenuItem("Paste"));

		mSettings.add(m_miCursorColor=new MenuItem("Cursor Color"));
		mSettings.add(m_miTabSize=new MenuItem("Tab Size"));
		mSettings.add(m_miJoinAddr=new MenuItem("Join Address"));

		mbar.add(mFile);
		mbar.add(mEdit);
		mbar.add(mSettings);
		mbar.add(mHelp);
		mbar.setHelpMenu(mHelp);

		setMenuBar(mbar);
	}

	//------------------------------------------------------------------
	// Handle an event
	public boolean handleEvent(Event evt) {

		//if (Event.MOUSE_MOVE!=evt.id) {
		//	System.out.println("In SEdit.handleEvent()"); 
		//	System.out.flush();
		//}
		//EventPrinter.printEvent(evt);

		if (evt.target instanceof MenuItem) {
			// Menu events
			MenuItem miTarget=(MenuItem)evt.target;
			m_statusbar.setMessage(miTarget.getLabel());
			if (miTarget==m_miNew) {
				newCmd();
			} else if (miTarget==m_miOpen) {
				openCmd();
			} else if (miTarget==m_miJoin) {
				new SEditJoinDialog(this);
				// This dialog is modal. When it is dismissed, we will get more events.
			} else if (miTarget==m_miClose) {
				closeCmd();
			} else if (miTarget==m_miSaveAs) {
				saveAsCmd();
			} else if (miTarget==m_miQuit) {
				quitCmd();
			} else if (miTarget==m_miAbout) {
				new SEditAboutDialog(this);
				// This dialog is modal. When it is dismissed, we will get more events.
			} else if (miTarget==m_miCut) {
				m_statusbar.setMessage("Cut NYI");
			} else if (miTarget==m_miCopy) {
				m_statusbar.setMessage("Copy NYI");
			} else if (miTarget==m_miPaste) {
				m_statusbar.setMessage("Paste NYI");
			} else if (miTarget==m_miCursorColor) {
				m_statusbar.setMessage("Cursor Color NYI");
			} else if (miTarget==m_miTabSize) {
				m_statusbar.setMessage("Tab Size NYI");
			} else if (miTarget==m_miJoinAddr) {
				Assert.isNotNull(m_seNDO);
				m_statusbar.setMessage("Join Address: "+m_seNDO.joinAddr());
			} else {
				Assert.error("Unknown menu!");
			}
			return true;

		} else if (evt.target instanceof TextField) {
			// We must do this so we can pump our child dialogs
			return false;

		} else if (evt.target instanceof Scrollbar) {
			//System.out.println("Scrollbar message");
			return false;

		} else if (evt.target instanceof SEditJoinDialog) {
			joinCmd((SEditJoinDialog)evt.target);

		} else if (evt.target==this || evt.target instanceof Canvas || evt.target instanceof Panel) {
			// Window events
			// we don't care which component they're from
			switch (evt.id) {
			case Event.WINDOW_DESTROY:
				quitCmd();
				return true;
			case Event.KEY_PRESS:
				if (null!=m_seNDO) {
					if ((evt.key>=' ' && evt.key<='~') || '\t'==evt.key) {
						insertChar(evt);
					} else if (8==evt.key || 127==evt.key) {
						deleteChar(evt);
					} else if (10==evt.key || 13==evt.key) {
						insertLine(evt);
					} else {
						m_statusbar.setMessage("Key "+evt.key+" pressed.");
					}
					return true;
				}
				return true;
			case Event.KEY_ACTION:
				if (null!=m_seNDO) {
					if (Event.UP==evt.key || Event.DOWN==evt.key 
						|| Event.LEFT==evt.key || Event.RIGHT==evt.key
						|| Event.END==evt.key || Event.HOME==evt.key) {
						moveCursor(evt);
						return true;
					}
				}
			case Event.MOUSE_ENTER:
				// Win95 bug - never revceive focus from window manager!
				// so, request as soon as the mouse is over us.
				//requestFocus();
				//return true;
			}			 
		} else {
			Assert.error("Event in unknown component: "+evt);
		}

		return false;
	}

	//------------------------------------------------------------------
	// insert a character into a line
	private void insertLine(Event event) {
		Assert.isTrue(10==event.key || 13==event.key);

		// Convenience stuff
		SEditSCursor sscMyCursor=m_seNDO.cursor();
		SharedVector svLineList=m_seNDO.lineList();
		SharedContainer scCursorList=m_seNDO.cursorList();
		boolean bRetry;
		do {
			bRetry=false;

			// Lock what we need
			int nLine=0;
			SharedString sszOld=null;
			while (true) {
				nLine=sscMyCursor.getLine();
				sszOld=m_seNDO.getLineObject(nLine);
				m_seNDO.startLock();
				scCursorList.lockAll();
				sszOld.lock();
				svLineList.lock();
				try {
					m_seNDO.completeLock();
					break;
				} catch (FieldDeletedException e) {
					// That's OK, try again.
				}
			}

			// make sure we get the line that the cursor is ACTUALLY on.
			int nCursLine=sscMyCursor.getLine();
			int nCursCol=sscMyCursor.getCol();
			if (nCursLine!=nLine) {
				bRetry=true;
			} else {
				// First split the line at the cursor
				String szOldLine=sszOld.get();
				String szNewLine="";
				if (nCursCol<szOldLine.length()) {
					szNewLine=szOldLine.substring(nCursCol);
					szOldLine=szOldLine.substring(0,nCursCol);
				}
				sszOld.set(szOldLine);
				SharedString sszNew=new SharedString(svLineList);
				sszNew.set(szNewLine);
				if (nCursLine+1!=svLineList.size()) {
					svLineList.moveLastElementTo(nCursLine+1);
				}

				// Now, update cursors:
				for (int nIndex=scCursorList.nextID()-1; nIndex>=0; nIndex--) {
					try {
						SEditSCursor sscTemp=(SEditSCursor)scCursorList.field(nIndex);
						// skip ourselves
						if (sscTemp==sscMyCursor) {
							continue;
						}
						int nTempLine=sscTemp.getLine();
						if (nTempLine>nCursLine) {
							sscTemp.setLine(nTempLine+1);
						} else if (nTempLine==nCursLine) {
							int nTempCol=sscTemp.getCol();
							if (nTempCol>=nCursCol) {
								sscTemp.setLine(nTempLine+1);
								nTempCol-=nCursCol;
								sscTemp.setCol(nTempCol);
								sscTemp.setVirtCol(nTempCol);
							}
						}
					} catch (FieldDeletedException e) {
					}
				}

				// Now update our cursor
				sscMyCursor.setLine(nCursLine+1);
				sscMyCursor.setCol(0);
				sscMyCursor.setVirtCol(0);

				sszNew.unlock();
			}

			scCursorList.unlockAll();
			sszOld.unlock();
			svLineList.unlock();
		} while (true==bRetry);

	}

	//------------------------------------------------------------------
	// insert a character into a line
	private void insertChar(Event event) {
		Assert.isTrue((event.key>=' ' && event.key<='~') || '\t'==event.key);

		// Convenience stuff
		SEditSCursor sscMyCursor=m_seNDO.cursor();
		SharedContainer scCursorList=m_seNDO.cursorList();
		boolean bRetry;
		do {
			bRetry=false;

			// Lock what we need
			int nLine=0;
			SharedString sszLine=null;
			while (true) {
				nLine=sscMyCursor.getLine();
				sszLine=m_seNDO.getLineObject(nLine);
				m_seNDO.startLock();
				scCursorList.lockAll();
				sszLine.lock();
				try {
					m_seNDO.completeLock();
					break;
				} catch (FieldDeletedException e) {
					// That's OK, try again.
				}
			}

			// make sure we get the line that the cursor is ACTUALLY on.
			int nCursLine=sscMyCursor.getLine();
			int nCursCol=sscMyCursor.getCol();
			if (nCursLine!=nLine) {
				bRetry=true;
			} else {
				sszLine.set((new StringBuffer(sszLine.get())).insert(nCursCol, (char)event.key).toString());
				// Now, update cursors:
				for (int nIndex=scCursorList.nextID()-1; nIndex>=0; nIndex--) {
					try {
						SEditSCursor sscTemp=(SEditSCursor)scCursorList.field(nIndex);
						// skip ourselves
						if (sscTemp==sscMyCursor) {
							continue;
						}
						int nTempLine=sscTemp.getLine();
						if (nTempLine==nCursLine) {
							int nTempCol=sscTemp.getCol();
							if (nTempCol>=nCursCol) {
								sscTemp.setCol(nTempCol+1);
								sscTemp.setVirtCol(nTempCol+1);
							}
						}
					} catch (FieldDeletedException e) {
					}
				}

				// Now update our cursor
				sscMyCursor.setCol(nCursCol+1);
				sscMyCursor.setVirtCol(nCursCol+1);
			}
			scCursorList.unlockAll();
			sszLine.unlock();
		} while (true==bRetry);
	}

	//------------------------------------------------------------------
	// delete a character
	private void deleteChar(Event event) {
		Assert.isTrue(8==event.key || 127==event.key);
		
		while (true) {
			SEditSCursor sscMyCursor=m_seNDO.cursor();
			int nCursLine=sscMyCursor.getLine();
			int nCursCol=sscMyCursor.getCol();
			if ((0==nCursCol && 8==event.key)
				|| (127==event.key && nCursLine<m_seNDO.numLines() 
				&& m_seNDO.getLine(nCursLine).length()==nCursCol)) {
				if (true==deleteOuterChar(event)) {
					break;
				}
			} else {
				if (true==deleteInnerChar(event)) {
					break;
				}
			}
		}
	}

	//------------------------------------------------------------------
	// delete a character
	private boolean deleteOuterChar(Event event) {
		Assert.isTrue(8==event.key || 127==event.key);
		// Convenience stuff
		SEditSCursor sscMyCursor=m_seNDO.cursor();
		SharedVector svLineList=m_seNDO.lineList();
		SharedContainer scCursorList=m_seNDO.cursorList();

		if ((0==sscMyCursor.getLine() && 8==event.key) 
			|| (127==event.key && sscMyCursor.getLine()==m_seNDO.numLines()-1)) {
			return true;
		}

		boolean bRetVal=true;
		boolean bRetry;
		do {
			bRetry=false;

			// Lock what we need
			int nLine=0;
			SharedString sszOldTop=null;
			SharedString sszOldBottom=null;
			while (true) {
				nLine=sscMyCursor.getLine();
				if (8==event.key) {
					sszOldTop=m_seNDO.getLineObject(nLine-1);
					sszOldBottom=m_seNDO.getLineObject(nLine);
				} else {
					sszOldTop=m_seNDO.getLineObject(nLine);
					sszOldBottom=m_seNDO.getLineObject(nLine+1);
				}
				m_seNDO.startLock();
				scCursorList.lockAll();
				sszOldTop.lock();
				sszOldBottom.lock();
				svLineList.lock();
				try {
					m_seNDO.completeLock();
					break;
				} catch (FieldDeletedException e) {
					// That's OK, try again.
				}
			}

			// make sure we get the line that the cursor is ACTUALLY on.
			int nCursLine=sscMyCursor.getLine();
			int nCursCol=sscMyCursor.getCol();
			if (nCursLine!=nLine) {
				bRetry=true;
			} else 	if ((0==nCursCol && 8==event.key)
					|| (127==event.key && nCursLine<m_seNDO.numLines() 
					&& m_seNDO.getLine(nCursLine).length()==nCursCol)) {

				// join the lines
				String szOldTop=sszOldTop.get();
				String szNewTop=szOldTop+sszOldBottom.get();
				sszOldTop.set(szNewTop);
				
				if (8==event.key) {
					svLineList.removeElementAt(nLine);
					--nCursLine;
					nCursCol=szOldTop.length();
				} else {
					svLineList.removeElementAt(nLine+1);
				}
				// Now, update cursors:
				for (int nIndex=scCursorList.nextID()-1; nIndex>=0; nIndex--) {
					try {
						SEditSCursor sscTemp=(SEditSCursor)scCursorList.field(nIndex);
						// skip ourselves
						if (sscTemp==sscMyCursor) {
							continue;
						}
						int nTempLine=sscTemp.getLine();
						if (nTempLine>nCursLine+1) {
							sscTemp.setLine(nTempLine-1);
						} else if (nTempLine==nCursLine+1) {
							int nTempCol=sscTemp.getCol();
							nTempCol+=szOldTop.length();
							sscTemp.setCol(nTempCol);
							sscTemp.setVirtCol(nTempCol);
						}
					} catch (FieldDeletedException e) {
					}
				}

				// Now update our cursor
				sscMyCursor.setLine(nCursLine);
				sscMyCursor.setCol(nCursCol);
				sscMyCursor.setVirtCol(nCursCol);

			} else {
				bRetVal=false;
			}

			scCursorList.unlockAll();
			sszOldTop.unlock();
			svLineList.unlock();
		} while (true==bRetry);

		return bRetVal;
	}

	//------------------------------------------------------------------
	// delete a character
	private boolean deleteInnerChar(Event event) {
		Assert.isTrue(8==event.key || 127==event.key);

		// Convenience stuff
		SEditSCursor sscMyCursor=m_seNDO.cursor();
		SharedContainer scCursorList=m_seNDO.cursorList();
		boolean bRetry;
		boolean bRetVal=true;
		do {
			bRetry=false;

			// Lock what we need
			int nLine=0;
			SharedString sszLine=null;
			while (true) {
				nLine=sscMyCursor.getLine();
				sszLine=m_seNDO.getLineObject(nLine);
				m_seNDO.startLock();
				scCursorList.lockAll();
				sszLine.lock();
				try {
					m_seNDO.completeLock();
					break;
				} catch (FieldDeletedException e) {
					// That's OK, try again.
				}
			}

			// make sure we get the line that the cursor is ACTUALLY on.
			int nCursLine=sscMyCursor.getLine();
			int nCursCol=sscMyCursor.getCol();
			if (nCursLine!=nLine) {
				bRetry=true;
			} else {
				String szSource=sszLine.get();
				String szTarget;
				if (8==event.key) {
					if (0!=nCursCol) {
						szTarget=szSource.substring(0,nCursCol-1)+szSource.substring(nCursCol);
						sszLine.set(szTarget);
						--nCursCol;
						sscMyCursor.setCol(nCursCol);
						sscMyCursor.setVirtCol(nCursCol);
					} else {
						// Nope - can't do it
						bRetVal=false;
					}
				} else {
					if (szSource.length()!=nCursCol) {
						szTarget=szSource.substring(0,nCursCol)+szSource.substring(nCursCol+1);
						sszLine.set(szTarget);
					} else {
						// Nope - can't do it
						bRetVal=false;
					}
				}

				if (true==bRetVal) {
					// Now, update cursors:
					for (int nIndex=scCursorList.nextID()-1; nIndex>=0; nIndex--) {
						try {
							SEditSCursor sscTemp=(SEditSCursor)scCursorList.field(nIndex);
							// skip ourselves
							if (sscTemp==sscMyCursor) {
								continue;
							}
							int nTempLine=sscTemp.getLine();
							if (nTempLine==nCursLine) {
								int nTempCol=sscTemp.getCol();
								if (nTempCol>nCursCol) {
									sscTemp.setCol(nTempCol-1);
									sscTemp.setVirtCol(nTempCol-1);
								}
							}
						} catch (FieldDeletedException e) {
						}
					}
				}
			}
			scCursorList.unlockAll();
			sszLine.unlock();
		} while (true==bRetry);

		return bRetVal;
	}

	//------------------------------------------------------------------
	// move the cursor
	// BUGBUG: Deltas no longer needed
	private void moveCursor(Event event) {
		Assert.isTrue(Event.UP==event.key || Event.DOWN==event.key 
			|| Event.LEFT==event.key || Event.RIGHT==event.key
			|| Event.END==event.key || Event.HOME==event.key);
		SEditSCursor cursor=m_seNDO.cursor();
		m_seNDO.startLock();
		cursor.lock();
		try {
			m_seNDO.completeLock();
		} catch (FieldDeletedException e) {
			Assert.error("cursor deleted");
		}
		int nDeltaX=0;
		int nDeltaY=0;
		int nCursX=cursor.getCol();
		int nCursVX=cursor.getVirtCol();
		int nCursY=cursor.getLine();

		if (Event.UP==event.key) {
			if (nCursY>0) {
				nDeltaY=-1;
				--nCursY;
				cursor.setLine(nCursY);

				// adjust the cursor column
				int nLineLen=m_seNDO.getLine(nCursY).length();
				if (nLineLen<nCursVX) {
					nDeltaX=nLineLen-nCursX;
					nCursX=nLineLen;
				} else {
					nDeltaX=nCursVX-nCursX;
					nCursX=nCursVX;
				}
				cursor.setCol(nCursX);
			}
		} else if (Event.DOWN==event.key) {
			if (nCursY<m_seNDO.numLines()-1) {
				nDeltaY=1;
				++nCursY;
				cursor.setLine(nCursY);

				// adjust the cursor column
				int nLineLen=m_seNDO.getLine(nCursY).length();
				if (nLineLen<nCursVX) {
					nDeltaX=nLineLen-nCursX;
					nCursX=nLineLen;
				} else {
					nDeltaX=nCursVX-nCursX;
					nCursX=nCursVX;
				}
				cursor.setCol(nCursX);
			}
		} else if (Event.LEFT==event.key) {
			if (nCursX>0) {
				nDeltaX=-1;
				--nCursX;
				cursor.setCol(nCursX);
				cursor.setVirtCol(nCursX);
			} else if (nCursY>0) {
				nDeltaY=-1;
				--nCursY;
				cursor.setLine(nCursY);

				// adjust the cursor column
				int nLineLen=m_seNDO.getLine(nCursY).length();
				nDeltaX=nLineLen-nCursX;
				nCursX=nLineLen;
				cursor.setCol(nCursX);
				cursor.setVirtCol(nCursX);
			}
		} else if (Event.RIGHT==event.key) {
			int nLineLen=m_seNDO.getLine(nCursY).length();
			if (nCursX<nLineLen) {
				nDeltaX=1;
				++nCursX;
				cursor.setCol(nCursX);
				cursor.setVirtCol(nCursX);
			} else if (nCursY<m_seNDO.numLines()-1) {
				nDeltaY=1;
				++nCursY;
				cursor.setLine(nCursY);
				nDeltaX=-nCursX;
				nCursX=0;
				cursor.setCol(nCursX);
				cursor.setVirtCol(nCursX);
			}
		} else if (Event.HOME==event.key) {
			if (nCursX!=0) {
				nDeltaX=-nCursX;
				nCursX=0;
				cursor.setCol(nCursX);
				cursor.setVirtCol(nCursX);
			}
		} else if (Event.END==event.key) {
			int nLineLen=m_seNDO.getLine(nCursY).length();
			nDeltaX=nLineLen-nCursX;
			if (nDeltaX!=0) {
				nCursX=nLineLen;
				cursor.setCol(nCursX);
				cursor.setVirtCol(nCursX);
			}
		}
		cursor.unlock();

	}

	//------------------------------------------------------------------
	// create a new file 
	private void newCmd() {
		closeCmd();
		m_statusbar.setMessage("Opening new file");
		m_seNDO=new SEditNDO();

		// set the file name
		m_seNDO.startLock();
		m_seNDO.fileName().lock();
		try {
			m_seNDO.completeLock();
		} catch (FieldDeletedException e) {
			Assert.error("filename deleted");
		}
		m_seNDO.fileName().set("Unnamed");
		m_seNDO.fileName().unlock();

		commonNDOInit();
	}

	//------------------------------------------------------------------
	// quit editing this file
	private void closeCmd() {
		m_statusbar.setMessage("Closing current file");
		if (null!=m_pnlCenter) {
			remove(m_pnlCenter);
			m_pnlCenter=null;
		}
		add("Center", m_canvEmpty);
		validate();

		if (null!=m_seNDO) {
			m_seNDO.leave();
			m_seNDO=null;
		}

		setTitle(mc_szAppName);
		setMenusToNoFile();
	}

	//------------------------------------------------------------------
	// edit a remote file
	private void joinCmd(SEditJoinDialog sjd) {
		if (true==sjd.canceled()) {
			m_statusbar.setMessage("Join canceled");
		} else {
			closeCmd();
			String szName=sjd.getComputer().trim()+" "+sjd.getPort().trim();
			m_statusbar.setMessage("Joining \""+szName+"\"");
			m_seNDO=new SEditNDO(szName);
			commonNDOInit();
		}
	}

	//------------------------------------------------------------------
	// choose file to open
	private void openCmd() {
		FileDialog fd=new FileDialog(this, "Open", FileDialog.LOAD);
		fd.show();
		// wait for dialog to be dismissed - it's supposed to be modal
		if (null==fd.getFile()) {
			m_statusbar.setMessage("Open canceled");
			return;
		}

		// user selected a file - open it
		// first, close anything we had open
		closeCmd();

		// create a new NDO
		String szFullName=fd.getDirectory()+fd.getFile();
		m_statusbar.setMessage("Opening "+szFullName);
		m_seNDO=new SEditNDO();

		// read the file
		boolean bSuccess=true;
		String szErrMessage;
		try {
			FileInputStream fis=new FileInputStream(szFullName);
			bSuccess=m_seNDO.readFile(fis);
			szErrMessage="Error reading "+szFullName; // just in case
			fis.close();
		} catch (FileNotFoundException e) {
			bSuccess=false;
			szErrMessage="File not found "+szFullName;
		} catch (IOException e) {
			bSuccess=false;
			szErrMessage="Error reading "+szFullName;
		}
		if (false==bSuccess) {
			closeCmd();
			m_statusbar.setMessage(szErrMessage);
			return;
		}

		// set the file name
		m_seNDO.startLock();
		m_seNDO.fileName().lock();
		try {
			m_seNDO.completeLock();
		} catch (FieldDeletedException e) {
			Assert.error("filename deleted");
		}
		m_seNDO.fileName().set(fd.getFile());
		m_seNDO.fileName().unlock();

		commonNDOInit();

	}

	//------------------------------------------------------------------
	// Some stuff we do every time
	private void commonNDOInit() {

		String szName=m_seNDO.fileName().get();

		m_statusbar.setMessage("Opened "+szName);
		setMenusToFile();
		createCenterPanel();

		m_seNDO.lineList().addObserver(this);
		observeAll(m_seNDO.lineList());
		m_seNDO.cursorList().addObserver(this);
		observeAll(m_seNDO.cursorList());

		// Initialize the cursor
		m_nTextHOffset=0;
		m_nTextVOffset=0;
		SEditSCursor cursor=m_seNDO.cursor();
		m_seNDO.startLock();
		cursor.lock();
		try {
			m_seNDO.completeLock();
		} catch (FieldDeletedException e) {
			Assert.error("cursor deleted");
		}
		cursor.setCol(0);
		cursor.setLine(0);
		cursor.m_nCursLastXFile=0;
		cursor.m_nCursLastYFile=0;
		cursor.setFg(m_cCursFg.getRGB());
		cursor.setBg(m_cCursBg.getRGB());
		updateCursor(cursor);
		cursor.unlock();

		setTitle("["+szName+"] "+mc_szAppName);

	}

	//------------------------------------------------------------------
	// save a copy of this file
	private void saveAsCmd() {
		FileDialog fd=new FileDialog(this, "Save As", FileDialog.SAVE);
		fd.show();
		// wait for dialog to be dismissed - it's supposed to be modal
		if (null==fd.getFile()) {
			m_statusbar.setMessage("Save canceled");
			return;
		} 
		String szFullName=fd.getDirectory()+fd.getFile();
		m_statusbar.setMessage("Saveing  as "+szFullName);
		boolean bSuccess=true;
		String szErrMessage=null;
		try {
			FileOutputStream fos=new FileOutputStream(szFullName);
			bSuccess=m_seNDO.writeFile(fos);
			szErrMessage="Error writing "+szFullName; // just in case
			fos.close();
		} catch (IOException e) {
			bSuccess=false;
			szErrMessage="Error opening "+szFullName+" for writing";
		}

		if (false==bSuccess) {
			m_statusbar.setMessage(szErrMessage);
		} else {
			m_statusbar.setMessage("Saved "+szFullName);
		}
	}

	//------------------------------------------------------------------
	// Clean up ASAP.
	private void quitCmd() {
		closeCmd();
		hide();
		dispose();
		System.exit(0);
	}


	
	
	//------------------------------------------------------------------
	// enable/diable the appropriate menu settings
	private void setMenusToNoFile() {
		m_miClose.disable();
		m_miSaveAs.disable();
		m_miCut.disable();
		m_miCopy.disable();
		m_miPaste.disable();
		m_miJoinAddr.disable();
	}

	//------------------------------------------------------------------
	// enable/diable the appropriate menu settings
	private void setMenusToFile() {
		m_miClose.enable();
		m_miSaveAs.enable();
		//m_miCut.enable();
		//m_miCopy.enable();
		//m_miPaste.enable();
		m_miJoinAddr.enable();
	}


	
	//------------------------------------------------------------------
	// This is the callback for all observable objects
	public void update(Observable obsv, Object arg) {

		if (obsv instanceof SEditSCursor) {
			hideAllCursors();
			SEditSCursor cursor=(SEditSCursor)obsv;
			// was it our cursor?
			if (cursor==m_seNDO.cursor()) {
				int nDeltaX=cursor.getCol()-cursor.m_nCursLastXFile;
				int nDeltaY=cursor.getLine()-cursor.m_nCursLastYFile;
				if (nDeltaX!=0 || nDeltaY!=0) {
					cursor.m_nCursLastXFile+=nDeltaX;
					cursor.m_nCursLastYFile+=nDeltaY;
					matchScreenToCursorMove(nDeltaX, nDeltaY);
				}
			}
			updateAllCursors();

		} else if (obsv==m_seNDO.lineList()) {
			observeAll(m_seNDO.lineList());
			hideAllCursors();
			updateTextFully();
			updateAllCursors();
		} else if (obsv==m_seNDO.cursorList()) {
			observeAll(m_seNDO.cursorList());
		} else if (obsv instanceof SharedString) {
			hideAllCursors();
			updateLine((SharedString)obsv);
			updateAllCursors();
		}
	}


	//------------------------------------------------------------------
	// completely fill the text grid
	private void observeAll(SharedContainer sc) {
		for (int nIndex=sc.nextID()-1; nIndex>=0; nIndex--) {
			try {
				sc.field(nIndex).addObserver(this);
			} catch (FieldDeletedException e) {
			}
		}
	}

	//------------------------------------------------------------------
	// completely fill the text grid
	private void updateTextFully() {
		int nVOffset=m_seNDO.cursor().getLine()-m_nTextVOffset;
		int nHOffset=m_seNDO.cursor().getCol()-m_nTextHOffset;
		int nWidth=m_textgrid.getGridSize().width;

		m_textgrid.clear();

		for (int nYIndex=m_textgrid.getGridSize().height-1; nYIndex>=0; nYIndex--) {
			String szLine;
			String szTemp;
			if (nYIndex+nVOffset<m_seNDO.numLines()) {
				szLine=m_seNDO.getLine(nYIndex+nVOffset);
			} else {
				szLine="";
			}

			if (szLine.length()-nHOffset<=0) {
				continue;
			} else if (szLine.length()-nHOffset<nWidth) {
				szTemp=szLine.substring(nHOffset);
			} else {
				szTemp=szLine.substring(nHOffset, nHOffset+nWidth);
			}
			m_textgrid.print(0, nYIndex, szTemp);
		}
	}

	//------------------------------------------------------------------
	// Update the line on the text grid that matches this one.
	private void updateLine(SharedString ss) {
		int nVOffset=m_seNDO.cursor().getLine()-m_nTextVOffset;
		int nHOffset=m_seNDO.cursor().getCol()-m_nTextHOffset;
		int nWidth=m_textgrid.getGridSize().width;

		for (int nYIndex=m_textgrid.getGridSize().height-1; nYIndex>=0; nYIndex--) {
			if (nYIndex+nVOffset>=m_seNDO.numLines()) {
				continue;
			}
			if (ss==m_seNDO.getLineObject(nYIndex+nVOffset)) {
				// we found the line
				String szLine=ss.get();
				String szTemp;
				if (szLine.length()-nHOffset<=0) {
					break;
				} else if (szLine.length()-nHOffset<nWidth) {
					szTemp=szLine.substring(nHOffset);
				} else {
					szTemp=szLine.substring(nHOffset, nHOffset+nWidth);
				}
				m_textgrid.clearLine(nYIndex);
				m_textgrid.print(0, nYIndex, szTemp);
				break;
			}
		}
	}
	//------------------------------------------------------------------
	// reprint this line
	private void updateCurLine() {
		int nHOffset=m_seNDO.cursor().getCol()-m_nTextHOffset;
		int nWidth=m_textgrid.getGridSize().width;

		String szLine;
		String szTemp;
		if (m_seNDO.cursor().getLine()<m_seNDO.numLines()) {
			szLine=m_seNDO.getLine(m_seNDO.cursor().getLine());
		} else {
			szLine="";
		}

		if (szLine.length()-nHOffset<=0) {
			return;
		} else if (szLine.length()-nHOffset<nWidth) {
			szTemp=szLine.substring(nHOffset);
		} else {
			szTemp=szLine.substring(nHOffset, nHOffset+nWidth);
		}
		m_textgrid.clearLine(m_nTextVOffset);
		m_textgrid.print(0, m_nTextVOffset, szTemp);
	}

	//------------------------------------------------------------------
	// reprint specific line
	private void updateScreenLine(int nLine) {
		int nVOffset=m_seNDO.cursor().getLine()-m_nTextVOffset;
		int nHOffset=m_seNDO.cursor().getCol()-m_nTextHOffset;
		int nWidth=m_textgrid.getGridSize().width;

		//m_textgrid.clearLine(nLine);

		String szLine;
		String szTemp;
		if (nLine+nVOffset<m_seNDO.numLines()) {
			szLine=m_seNDO.getLine(nLine+nVOffset);
		} else {
			szLine="";
		}

		if (szLine.length()-nHOffset<=0) {
			return;
		} else if (szLine.length()-nHOffset<nWidth) {
			szTemp=szLine.substring(nHOffset);
		} else {
			szTemp=szLine.substring(nHOffset, nHOffset+nWidth);
		}
		m_textgrid.print(0, nLine, szTemp);
	}

	//------------------------------------------------------------------
	// show a cursor
	private void updateCursor(SEditSCursor cursor) {
		hideCursor(cursor);
		SEditSCursor sscOurCursor=m_seNDO.cursor();
		if (cursor.getLine()>=m_seNDO.numLines()) {
			return;
		}
		String szLine=m_seNDO.getLine(cursor.getLine());
		if (cursor.getCol()>=szLine.length()) {
			cursor.m_chCursLast=' ';
		} else {
			cursor.m_chCursLast=szLine.charAt(cursor.getCol());
		}
		cursor.m_nCursLastYScreen=m_nTextVOffset-sscOurCursor.getLine()+cursor.getLine();
		cursor.m_nCursLastXScreen=m_nTextHOffset-sscOurCursor.getCol()+cursor.getCol();
		if (cursor.m_nCursLastYScreen<0 || 
			cursor.m_nCursLastYScreen>=m_textgrid.getGridSize().height ||
			cursor.m_nCursLastXScreen<0 || 
			cursor.m_nCursLastXScreen>=m_textgrid.getGridSize().width) {
			// cursor is off the screen
		} else {
			m_textgrid.setChar(cursor.m_nCursLastXScreen, cursor.m_nCursLastYScreen, 
				cursor.m_chCursLast, cursor.m_cCursFg, cursor.m_cCursBg);
			cursor.m_bCursVisible=true;
		}
	}

	//------------------------------------------------------------------
	// hide a cursor
	private void hideCursor(SEditSCursor cursor) {
		if (true==cursor.m_bCursVisible) {
			m_textgrid.setChar(cursor.m_nCursLastXScreen, cursor.m_nCursLastYScreen, 
				cursor.m_chCursLast, null, null);
		}
		cursor.m_bCursVisible=false;
	}

	//------------------------------------------------------------------
	// hide the cursors
	private void hideAllCursors() {
		SharedContainer scCursorList=m_seNDO.cursorList();
		for (int nIndex=scCursorList.nextID()-1; nIndex>=0; nIndex--) {
			try {
				SEditSCursor sscTemp=(SEditSCursor)scCursorList.field(nIndex);
				hideCursor(sscTemp);
			} catch (FieldDeletedException e) {
			}
		}
	}

	//------------------------------------------------------------------
	// update the cursors
	private void updateAllCursors() {
		SharedContainer scCursorList=m_seNDO.cursorList();
		for (int nIndex=scCursorList.nextID()-1; nIndex>=0; nIndex--) {
			try {
				SEditSCursor sscTemp=(SEditSCursor)scCursorList.field(nIndex);
				updateCursor(sscTemp);
			} catch (FieldDeletedException e) {
			}
		}
	}

	//------------------------------------------------------------------
	// Now, correct the screen position and screen cursor
	private void matchScreenToCursorMove(int nDeltaX, int nDeltaY) {
		int nVScroll=0;
		int nHScroll=0;
		if (nDeltaY<0) {
			if (m_nTextVOffset+nDeltaY<0) {
				nVScroll=m_nTextVOffset+nDeltaY;
				m_nTextVOffset=0;
			} else {
				m_nTextVOffset+=nDeltaY;
			}
		} else if (nDeltaY>0) {
			if (m_textgrid.getGridSize().height-1-m_nTextVOffset-nDeltaY<0) {
				nVScroll=1-m_textgrid.getGridSize().height+m_nTextVOffset+nDeltaY;
				m_nTextVOffset=m_textgrid.getGridSize().height-1;
			} else {
				m_nTextVOffset+=nDeltaY;
			}
		}
		if (nDeltaX<0) {
			if (m_nTextHOffset+nDeltaX<0) {
				nHScroll=m_nTextHOffset+nDeltaX;
				m_nTextHOffset=0;
			} else {
				m_nTextHOffset+=nDeltaX;
			}
		} else if (nDeltaX>0) {
			if (m_textgrid.getGridSize().width-1-m_nTextHOffset-nDeltaX<0) {
				nHScroll=1-m_textgrid.getGridSize().width+m_nTextHOffset+nDeltaX;
				m_nTextHOffset=m_textgrid.getGridSize().width-1;
			} else {
				m_nTextHOffset+=nDeltaX;
			}
		}

		if (nVScroll>=m_textgrid.getGridSize().height-1
			|| -nVScroll>=m_textgrid.getGridSize().height-1
			|| nHScroll>=m_textgrid.getGridSize().width-1
			|| -nHScroll>=m_textgrid.getGridSize().width-1) {
			// must update the whole screen
			updateTextFully();

		} else {
			// Scroll vertically
			if (nVScroll<0) {
				m_textgrid.scrollDown(-nVScroll);
			} else if (nVScroll>0) {
				m_textgrid.scrollUp(nVScroll);
			}

			// scroll and update horizontally
			scrollHoriz(nHScroll);

			// update vertically
			if (nVScroll<0) {
				for (int nIndex=0; nIndex<-nVScroll; nIndex++) {
					updateScreenLine(nIndex);
				}
			} else if (nVScroll>0) {
				for (int nYIndex=m_textgrid.getGridSize().height-1; nVScroll>0; --nYIndex, --nVScroll) {
					updateScreenLine(nYIndex);
				}
			}
		}
	}

	//------------------------------------------------------------------
	// scroll the text up 
	private void scrollVert(int n) {
		if (n<0) {
			m_textgrid.scrollDown(-n);
			for (int nIndex=0; nIndex<-n; nIndex++) {
				updateScreenLine(nIndex);
			}
		} else if (n>0) {
			m_textgrid.scrollUp(n);
			for (int nYIndex=m_textgrid.getGridSize().height-1; n>0; --nYIndex, --n) {
				updateScreenLine(nYIndex);
			}
		}
	}

	//------------------------------------------------------------------
	// scroll the text sideways
	private void scrollHoriz(int n) {
		int nVOffset=m_seNDO.cursor().getLine()-m_nTextVOffset;
		int nHOffset=m_seNDO.cursor().getCol()-m_nTextHOffset;
		int nWidth=m_textgrid.getGridSize().width;

		if (n<0) {
			m_textgrid.scrollRight(-n);

			nWidth=-n;
			for (int nYIndex=m_textgrid.getGridSize().height-1; nYIndex>=0; nYIndex--) {
				String szLine;
				String szTemp;
				if (nYIndex+nVOffset<m_seNDO.numLines()) {
					szLine=m_seNDO.getLine(nYIndex+nVOffset);
				} else {
					szLine="";
				}
				
				if (szLine.length()-nHOffset<=0) {
					continue;
				} else if (szLine.length()-nHOffset<nWidth) {
					szTemp=szLine.substring(nHOffset);
				} else {
					szTemp=szLine.substring(nHOffset, nHOffset+nWidth);
				}
				m_textgrid.print(0, nYIndex, szTemp);
			}
		} else if (n>0) {
			m_textgrid.scrollLeft(n);

			int nOffset=nWidth-n;
			nHOffset+=nOffset;
			nWidth=n;
			for (int nYIndex=m_textgrid.getGridSize().height-1; nYIndex>=0; nYIndex--) {
				String szLine;
				String szTemp;
				if (nYIndex+nVOffset<m_seNDO.numLines()) {
					szLine=m_seNDO.getLine(nYIndex+nVOffset);
				} else {
					szLine="";
				}
				
				if (szLine.length()-nHOffset<=0) {
					continue;
				} else if (szLine.length()-nHOffset<nWidth) {
					szTemp=szLine.substring(nHOffset);
				} else {
					szTemp=szLine.substring(nHOffset, nHOffset+nWidth);
				}
				m_textgrid.print(nOffset, nYIndex, szTemp);
			}
		}
	}

}
