//----------------------------------------------------------------------
// SEditJoinDialog
// by Louis Thomas
// Last modified: 6-3-97
package cs141b.sedit2;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Event;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;

//----------------------------------------------------------------------
// The Join... dialog class
public class SEditJoinDialog extends Dialog {

	//------------------------------------------------------------------
	// Member Variables
	private Button m_btnOK;
	private Button m_btnCancel;
	private TextField m_tfComputer;
	private TextField m_tfPort;
	private boolean m_bCanceled;
	private Component m_compCurrentFocus;

	//------------------------------------------------------------------
	// Constructor
	public SEditJoinDialog(SEdit parent) {
		super(parent, "Join", true);

		Panel pnlBottom=new Panel();
		m_btnOK=new Button("  OK  ");
		pnlBottom.add(m_btnOK);
		m_btnCancel=new Button("Cancel");
		pnlBottom.add(m_btnCancel);

		Panel pnlCenter=new Panel();
		GridBagLayout gbl=new GridBagLayout();
		GridBagConstraints gbc=new GridBagConstraints();
		pnlCenter.setLayout(gbl);

		m_tfComputer=new TextField(30);
		m_tfPort=new TextField(30);
		Label lblComputer=new Label("Computer: ", Label.RIGHT);
		Label lblPort=new Label("Port: ", Label.RIGHT);

		gbc.anchor=GridBagConstraints.EAST;
		gbc.gridx=0;
		gbc.gridy=0;
		gbl.setConstraints(lblComputer, gbc);
		pnlCenter.add(lblComputer);

		gbc.anchor=GridBagConstraints.WEST;
		gbc.gridx=1;
		gbc.gridy=0;
		gbl.setConstraints(m_tfComputer, gbc);
		pnlCenter.add(m_tfComputer);

		gbc.anchor=GridBagConstraints.EAST;
		gbc.gridx=0;
		gbc.gridy=1;
		gbl.setConstraints(lblPort, gbc);
		pnlCenter.add(lblPort);

		gbc.anchor=GridBagConstraints.WEST;
		gbc.gridx=1;
		gbc.gridy=1;
		gbl.setConstraints(m_tfPort, gbc);
		pnlCenter.add(m_tfPort);

		add("Center", pnlCenter);
		add("South", pnlBottom);

		setBackground(Color.lightGray);
		setResizable(false);
		resize(300,150);
		//pack();
		show();
	}

	//------------------------------------------------------------------
	// Handle show
	public void show() {
		m_compCurrentFocus=m_tfComputer;
		m_compCurrentFocus.requestFocus();
		super.show();
		m_compCurrentFocus.requestFocus();
	}

	//------------------------------------------------------------------
	// return the size of the inside border
	public Insets insets() {
		Insets ins=super.insets();
		return new Insets(ins.top+10, ins.left+10, ins.bottom+10, ins.right+10);
	}

	//------------------------------------------------------------------
	// Handle an event
	public boolean handleEvent(Event evt) {

		//if (Event.MOUSE_MOVE!=evt.id) {
		//	System.out.println("In SEditJoinDialog.handleEvent()"); 
		//	System.out.flush();
		//}
		//EventPrinter.printEvent(evt);

		boolean bShift=0!=(evt.modifiers&Event.SHIFT_MASK);

		if (evt.target==this) {
			switch (evt.id) {
			case Event.WINDOW_DESTROY:
				m_bCanceled=true;
				quit();
				break;
			case Event.MOUSE_ENTER:
			case Event.GOT_FOCUS:
				m_compCurrentFocus.requestFocus();
			}		
		} else if (evt.target==m_btnOK || evt.target==m_btnCancel) {
			switch(evt.id) {

			case Event.GOT_FOCUS:
				m_compCurrentFocus=(Component)evt.target;
				break;

			case Event.KEY_PRESS:
				if (13==evt.key || 10==evt.key || 32==evt.key) {
					// fall through
				} else if (9==evt.key) { 
					if (true==bShift) {
						if (evt.target==m_btnOK) {
							m_compCurrentFocus=m_tfPort;
						} else {
							m_compCurrentFocus=m_btnOK;
						}
					} else {
						if (evt.target==m_btnOK) {
							m_compCurrentFocus=m_btnCancel;
						} else {
							m_compCurrentFocus=m_tfComputer;
						}
					}
					m_compCurrentFocus.requestFocus();
					break;
				} else {
					break;
				}

			case Event.ACTION_EVENT:
				if (evt.target==m_btnOK) {
					m_bCanceled=false;
				} else {
					m_bCanceled=true;
				}
				quit();
				break;
			}
		} else if (evt.target==m_tfComputer || evt.target==m_tfPort) {
			boolean bDontPass=false;
			switch(evt.id) {

			case Event.GOT_FOCUS:
				m_compCurrentFocus=(Component)evt.target;
				// AWT bug: mouse clicks in these components 
				// don't always produce this event
				break;

			case Event.KEY_PRESS:
				m_compCurrentFocus=(Component)evt.target;
				if (27==evt.key) {
					m_bCanceled=true;
					quit();
					bDontPass=true;
				} else if (13==evt.key || 10==evt.key || 9==evt.key) {
					if (true==bShift && 9==evt.key) {
						if (evt.target==m_tfComputer) {
							m_compCurrentFocus=m_btnCancel;
						} else {
							m_compCurrentFocus=m_tfComputer;
						}
					} else {
						if (evt.target==m_tfComputer) {
							m_compCurrentFocus=m_tfPort;
						} else {
							m_compCurrentFocus=m_btnOK;
						}
					}
					m_compCurrentFocus.requestFocus();
					bDontPass=true;
				}
				break;
			}

			if (false==bDontPass) {
				// explicitly pass this on
				return super.handleEvent(evt);
			}
		} else if (evt.target instanceof Panel || evt.target instanceof Label) {
			switch(evt.id) {
			case Event.GOT_FOCUS:
				m_compCurrentFocus.requestFocus();
				break;
			}
		} else {
			Assert.error("Message from unknown source:"+evt);
		}

		// By default, we will handle all events
		return true;
	}

	//------------------------------------------------------------------
	// Clean up ASAP - inform parent that we've been dismissed
	private void quit() {
		hide();
		dispose();
		getParent().postEvent(new Event(this, Event.ACTION_EVENT, null));
	}

	//------------------------------------------------------------------
	// accessor
	public boolean canceled() {
		return m_bCanceled;
	}

	//------------------------------------------------------------------
	// accessor
	public String getComputer() {
		return m_tfComputer.getText();
	}

	//------------------------------------------------------------------
	// accessor
	public String getPort() {
		return m_tfPort.getText();
	}
}
