//----------------------------------------------------------------------
// SEditNDO
// by Louis Thomas
// Last modified: 6-3-97

package cs141b.sedit2;

import cs141b.gndo.FieldDeletedException;
import cs141b.gndo.NDO;
import cs141b.gndo.SharedContainer;
import cs141b.gndo.SharedInt;
import cs141b.gndo.SharedVector;
import cs141b.gndo.SharedString;
import info.net.Place;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

//----------------------------------------------------------------------
// The NDO class for the editor
public class SEditNDO extends NDO {

	//------------------------------------------------------------------
	// Member Variables
	private SharedString m_sszFileName;
	private SharedVector m_svLineList;
	private SharedContainer m_scCursorList;

	private int m_nCursorID;
	private SEditSCursor m_sscOurCursor;


	//------------------------------------------------------------------
	// New NDO Constructor
	public SEditNDO() {
		super();
		init();
	}

	//------------------------------------------------------------------
	// Join NDO Constructor
	public SEditNDO(String szJoinAddr) {
		super(new Place(szJoinAddr.trim()+" inbox"));
		init();
	}

	//------------------------------------------------------------------
	// Initialization
	private void init() {

		// get ownership
		startLock();
		m_scCursorList.lock();
		try {
			completeLock();
		} catch (FieldDeletedException e) {
			Assert.error("CursorList deleted");
		}

		// create our cursor
		m_nCursorID=m_scCursorList.nextID();
		m_sscOurCursor=new SEditSCursor(m_scCursorList);

		m_sscOurCursor.unlock();
		m_scCursorList.unlock();

	}

	//------------------------------------------------------------------
	// Build the fields of this NDO
	public void makeFields() {
		m_sszFileName=new SharedString(this);
		m_scCursorList=new SharedContainer(this);
		m_svLineList=new SharedVector(this);
		new SharedString(m_svLineList);
	}

	//------------------------------------------------------------------
	// leave this NDO
	public void leave() {
	
		// remove our cursor
		startLock();
		m_sscOurCursor.lock();
		m_scCursorList.lock();
		try {
			completeLock();
		} catch (FieldDeletedException e) {
			Assert.error("cursor deleted");
		}
		m_scCursorList.removeField(m_nCursorID);
		m_scCursorList.unlock();

		super.leave();
	}

	//------------------------------------------------------------------
	// accessor
	public SharedString fileName() {
		return m_sszFileName;
	}

	//------------------------------------------------------------------
	// accessor
	public SharedContainer cursorList() {
		return m_scCursorList;
	}

	//------------------------------------------------------------------
	// accessor
	public SharedVector lineList() {
		return m_svLineList;
	}

	//------------------------------------------------------------------
	// accessor
	public SEditSCursor cursor() {
		return m_sscOurCursor;
	}

	//------------------------------------------------------------------
	// accessor
	public int numLines() {
		return m_svLineList.size();
	}

	//------------------------------------------------------------------
	// accessor
	public String joinAddr() {
		Place plInbox=where();
		return plInbox.host()+" "+plInbox.port();
	}

	//------------------------------------------------------------------
	// accessor
	public String getLine(int nLine) {
		Assert.isTrue(nLine>=0 && nLine<numLines());
		SharedString sszTemp=null;
		sszTemp=(SharedString)m_svLineList.elementAt(nLine);
		return sszTemp.get();
	}

	//------------------------------------------------------------------
	// accessor
	public SharedString getLineObject(int nLine) {
		Assert.isTrue(nLine>=0 && nLine<numLines());
		SharedString sszTemp=null;
		sszTemp=(SharedString)m_svLineList.elementAt(nLine);
		return sszTemp;
	}

	//------------------------------------------------------------------
	// Read a file into this NDO, erasing the old data
	public boolean readFile(InputStream is) {
		Assert.isNotNull(is);
		DataInputStream dis;
		if (is instanceof DataInputStream) {
			dis=(DataInputStream)is;
		} else {
			dis=new DataInputStream(is);
		}

		int nIndex;
		boolean bRetVal=true;

		// First, lock down everything
		while (true) {
			startLock();
			m_svLineList.lockAll();
			m_scCursorList.lockAll();
			try {
				completeLock();
				break;
			} catch(FieldDeletedException e) {
				// no problem - try again
			}
		}

		// set the cursors back to the origin.
		for (nIndex=0; nIndex<m_scCursorList.nextID(); nIndex++) {
			if (false==m_scCursorList.doesFieldExist(nIndex)) {
				continue;
			}
			SEditSCursor secTemp=null;
			try {
				secTemp=(SEditSCursor)m_scCursorList.field(nIndex);
			} catch (FieldDeletedException e) {
				Assert.error("Cursor deleted while we had it locked");
			}
			secTemp.setCol(0);
			secTemp.setLine(0);
		}
		
		// get rid of all the lines currently in the NDO
		while(m_svLineList.size()>0) {
			m_svLineList.removeElementAt(0);
		}
		
		// try to read the file
		while(true) {
			String szTemp=null;
			try {
				szTemp=dis.readLine();
			} catch (EOFException e) {
				// no problem
			} catch (IOException e) {
				// bad news
				bRetVal=false;
			}
			if (null==szTemp) {
				break;
			}
			SharedString sszTemp=new SharedString(m_svLineList);
			sszTemp.set(szTemp);
		}

		// all done
		unlockAll();
		return bRetVal;
	}

	//------------------------------------------------------------------
	// write a file from this NDO, not locking the data
	public boolean writeFile(OutputStream os) {
		Assert.isNotNull(os);
		PrintStream ps;
		if (os instanceof PrintStream) {
			ps=(PrintStream)os;
		} else {
			ps=new PrintStream(os);
		}
		for (int nIndex=0; nIndex<m_svLineList.size(); ++nIndex) {
			ps.println(getLine(nIndex));
			if (true==ps.checkError()) {
				return false;
			}
		}
		ps.flush();

		return true;
	}
}

