//----------------------------------------------------------------------
// SEditSCursor
// by Louis Thomas
// Last modified: 6-3-97

package cs141b.sedit2;

import cs141b.gndo.FieldNotLockedException;
import cs141b.gndo.SharedContainer;
import cs141b.gndo.SharedVar;
import java.awt.Color;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

//----------------------------------------------------------------------
// The shared integer class
public class SEditSCursor extends SharedVar {

	//------------------------------------------------------------------
	// Member Variables
	private int m_nLine;
	private int m_nCol;
	private int m_nVirtCol;
	private int m_nFg;
	private int m_nBg;

	//------------------------------------------------------------------
	// Helper variables - these variables are not maintained over the network
	public Color m_cCursFg;
	public Color m_cCursBg;
	public boolean m_bCursVisible;
	public int m_nCursLastXScreen;
	public int m_nCursLastYScreen;
	public int m_nCursLastXFile;
	public int m_nCursLastYFile;
	public char m_chCursLast;

	//------------------------------------------------------------------
	// Constructor
	public SEditSCursor(SharedContainer scParent) {
		super(scParent);
		commonInit();
	}
	
	//------------------------------------------------------------------
	// Constructor
	public SEditSCursor() {
		// super() done automatically
		commonInit();
	}
	
	//------------------------------------------------------------------
	// do the standard initialization
	private void commonInit() {
		m_nLine=0;
		m_nCol=0;
		m_nVirtCol=0;
		m_nFg=Color.white.getRGB();
		m_nBg=Color.darkGray.getRGB();
		m_cCursFg=Color.white;
		m_cCursBg=Color.darkGray;
	}

	//------------------------------------------------------------------
	// <override> read the integer from a stream 
	public void readData(DataInputStream dis) throws IOException {
		Assert.isNotNull(dis);
		m_nLine=dis.readInt();
		m_nCol=dis.readInt();
		m_nVirtCol=dis.readInt();
		int nFg=dis.readInt();
		int nBg=dis.readInt();
		if (nFg!=m_nFg) {
			m_nFg=nFg;
			m_cCursFg=new Color(m_nFg);
		}
		if (nBg!=m_nBg) {
			m_nBg=nBg;
			m_cCursBg=new Color(m_nBg);
		}
	}
	
	//------------------------------------------------------------------
	// <override> Write the integer to a stream 
	public void writeData(DataOutputStream dos) throws IOException {
		Assert.isNotNull(dos);
		dos.writeInt(m_nLine);
		dos.writeInt(m_nCol);
		dos.writeInt(m_nVirtCol);
		dos.writeInt(m_nFg);
		dos.writeInt(m_nBg);
	}
	
	//------------------------------------------------------------------
	// Set a new value for the line
	public void setLine(int nLine) {
		if (!locked()) {
			throw new FieldNotLockedException("SEditSCursor not locked.");
		}
		if (m_nLine != nLine) {
			m_nLine=nLine;
			setModified();
		}
	}
	
	//------------------------------------------------------------------
	// Get the value of the line
	public int getLine() {
		return m_nLine;
	}

	//------------------------------------------------------------------
	// Set a new value for the column
	public void setCol(int nCol) {
		if (!locked()) {
			throw new FieldNotLockedException("SEditSCursor not locked.");
		}
		if (m_nCol != nCol) {
			m_nCol=nCol;
			setModified();
		}
	}
	
	//------------------------------------------------------------------
	// Get the value of the column
	public int getCol() {
		return m_nCol;
	}

	//------------------------------------------------------------------
	// Set a new value for the virtual column
	public void setVirtCol(int nVirtCol) {
		if (!locked()) {
			throw new FieldNotLockedException("SEditSCursor not locked.");
		}
		if (m_nVirtCol != nVirtCol) {
			m_nVirtCol=nVirtCol;
			setModified();
		}
	}
	
	//------------------------------------------------------------------
	// Get the value of the virtual column
	public int getVirtCol() {
		return m_nVirtCol;
	}

	//------------------------------------------------------------------
	// Set a new value for the foreground color
	public void setFg(int nFg) {
		if (!locked()) {
			throw new FieldNotLockedException("SEditSCursor not locked.");
		}
		if (m_nFg != nFg) {
			m_nFg=nFg;
			setModified();
			m_cCursFg=new Color(m_nFg);
		}
	}
	
	//------------------------------------------------------------------
	// Get the value of the foreground color
	public int getFg() {
		return m_nFg;
	}

	//------------------------------------------------------------------
	// Set a new value for the background color
	public void setBg(int nBg) {
		if (!locked()) {
			throw new FieldNotLockedException("SEditSCursor not locked.");
		}
		if (m_nBg != nBg) {
			m_nBg=nBg;
			setModified();
			m_cCursBg=new Color(m_nBg);
		}
	}
	
	//------------------------------------------------------------------
	// Get the value of the background color
	public int getBg() {
		return m_nBg;
	}

	//------------------------------------------------------------------
	// Get the value of the background color
	private void updateColors() {
		m_cCursFg=new Color(m_nFg);
		m_cCursBg=new Color(m_nBg);
	}
}
