//----------------------------------------------------------------------
// Status Bar
// by Louis Thomas
// Last modified: 5-25-97

package cs141b.sedit2;

import java.awt.Canvas;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Color;

//----------------------------------------------------------------------
// The Status Bar class
public class StatusBar extends Canvas {
	
	//------------------------------------------------------------------
	// Member Variables
	private String m_szMessage;
	private Font m_font;
	private Dimension m_dPrefSize;
	private int m_nVOffset;
	
	//------------------------------------------------------------------
	// Constructor
	public StatusBar() {
		super();

		m_font=new Font("TimesRoman", Font.PLAIN, 12);
		m_szMessage="";
		FontMetrics fm=getFontMetrics(m_font);
		m_dPrefSize=new Dimension(8+fm.stringWidth(m_szMessage), 8+fm.getAscent()+fm.getDescent());
		m_nVOffset=fm.getDescent()+2;

		setBackground(Color.lightGray);
		disable(); // don't want the focus!
	}
	
	//------------------------------------------------------------------
	// (override) Return the size of this component
	public Dimension minimumSize() {
		return m_dPrefSize;
	}
	
	//------------------------------------------------------------------
	// (override) Return the size of this component
	public Dimension preferredSize() {
		return m_dPrefSize;
	}
	
	//------------------------------------------------------------------
	// (override) Draw this component on the screen
	public void paint(Graphics g) {
		g.setFont(m_font);
		
		// draw top line
		g.setColor(Color.gray);
		g.drawLine(0, 0, size().width-1, 0);
		g.setColor(Color.white);
		g.drawLine(0, 1, size().width-1, 1);
		
		// draw 3D rect
		g.drawLine(0, size().height-1, size(). width-1, size().height-1);
		g.drawLine(size().width-1, 5, size().width-1, size().height-1);
		g.setColor(Color.darkGray);
		g.drawLine(0, 4, size().width-1, 4);
		g.drawLine(0, 4, 0, size().height-2);
		
		// display message
		g.setColor(Color.black);
		g.drawString(m_szMessage, 6, size().height-m_nVOffset);
	}
	
	//------------------------------------------------------------------
	// (override) Draw this component on the screen
	public void update(Graphics g) {
		g.setColor(getBackground());
		g.fillRect(1, 5, size().width-2, size().height-6);
		paint(g);
	}
	
	//------------------------------------------------------------------
	// Set the status message
	public void setMessage(String szMessage) {
		m_szMessage=szMessage;
		FontMetrics fm=getFontMetrics(m_font);
		m_dPrefSize=new Dimension(8+fm.stringWidth(m_szMessage), 8+fm.getAscent()+fm.getDescent());

		// update as soon as posible
		Graphics g=getGraphics();
		if (null==g) {
			repaint();
		} else {
			update(g);
		}
	}
	
	//------------------------------------------------------------------
	// Return the status message
	public String getMessage() {
		return m_szMessage;
	}
}
