//----------------------------------------------------------------------
// TextGrid
// by Louis Thomas
// Last modified: 6-3-97

package cs141b.sedit2;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;

//----------------------------------------------------------------------
// The Text Grid class
public class TextGrid extends Canvas {
	
	//------------------------------------------------------------------
	// Constants
	static private final Dimension mc_dCellSpc=new Dimension(0,0);
	
	//------------------------------------------------------------------
	// member variables
	private CharAttrib[][] m_caGrid;
	private char[] m_rgchTemp;
	private Font m_font;
	private Dimension m_dGrid;
	private Dimension m_dCell;
	private Dimension m_dCanv;
	private Dimension m_dOffs;

	private Image m_imOffscreen;
	private Graphics m_grOffscreen;

	private Color m_cDefaultBg;
	private Color m_cDefaultFg;
	private boolean m_bWrap;
	
	//------------------------------------------------------------------
	// Constructor
	public TextGrid(int nWidth, int nHeight, Font f, Color cFg, Color cBg) {
		super();
		Assert.isTrue(nWidth>0);
		Assert.isTrue(nHeight>0);

		m_font=f;
		FontMetrics fm=getFontMetrics(f);
		m_dGrid=new Dimension(nWidth, nHeight);
		m_dCell=new Dimension(fm.getMaxAdvance()+mc_dCellSpc.width, fm.getHeight()+mc_dCellSpc.height);
		m_dCanv=new Dimension(m_dGrid.width*m_dCell.width, m_dGrid.height*m_dCell.height);
		m_dOffs=new Dimension(mc_dCellSpc.width/2, mc_dCellSpc.height/2+fm.getAscent());

		m_cDefaultFg=cFg;
		m_cDefaultBg=cBg;
		setBackground(cBg);

		m_rgchTemp=new char[1];

		clear();
	}
	
	//------------------------------------------------------------------
	// clear the screen
	public void clear() {
		if (null==m_grOffscreen) {
			m_caGrid=new CharAttrib[m_dGrid.width][m_dGrid.height];
		} else {
			m_grOffscreen.setColor(m_cDefaultBg);
			m_grOffscreen.fillRect(0,0,m_dCanv.width,m_dCanv.height);
		}
		repaint();
	}

	//------------------------------------------------------------------
	// (override) When this is called, we can finally create the buffer.
	public void addNotify() {
		super.addNotify();
		if (null==m_imOffscreen) {
			createOffScreenBuffer();
		}
	}

	//------------------------------------------------------------------
	// (override) Return the size of this component
	public Dimension minimumSize() {
		return m_dCanv;
	}
	
	//------------------------------------------------------------------
	// (override) Return the size of this component
	public Dimension preferredSize() {
		return m_dCanv;
	}

	//------------------------------------------------------------------
	// (override) Draw this component on the screen
	public void paint(Graphics g) {
		synchronized(this) {
			Assert.isNotNull(m_imOffscreen);
			g.drawImage(m_imOffscreen, 0,0,this);
		}
	}
	
	//------------------------------------------------------------------
	// We can't create an image until peer has been created
	private void createOffScreenBuffer() {
		Assert.isNull(m_imOffscreen);
	
		// Create the image
		m_imOffscreen=createImage(m_dCanv.width, m_dCanv.height);
		if (m_imOffscreen==null) {
			throw new RuntimeException("TextGrid can't create offscreen image.");
		}
		
		// initialize it
		m_grOffscreen=m_imOffscreen.getGraphics();
		m_grOffscreen.setFont(m_font);
		m_grOffscreen.setColor(m_cDefaultBg);
		m_grOffscreen.fillRect(0,0,m_dCanv.width,m_dCanv.height);
		
		// draw all the cells, since some may already be changed
		for (int nXIndex=m_dGrid.width-1; nXIndex>=0; nXIndex--) {
			for (int nYIndex=m_dGrid.height-1; nYIndex>=0; nYIndex--) {
				CharAttrib caTemp=m_caGrid[nXIndex][nYIndex];
				if (null!=caTemp) {
					setChar(nXIndex, nYIndex, caTemp.ch, caTemp.fg, caTemp.bg);
				}
			}
		}
		m_caGrid=null;
	}
	
	
	//------------------------------------------------------------------
	// (override) Draw this component on the screen
	public void update(Graphics g) {
		// Do not paint background color (intentionally)
		paint(g);
	}
	
	//------------------------------------------------------------------
	// Change the character in a cell
	public synchronized void setChar(int nXPos, int nYPos, char ch, Color fg, Color bg) {
		if (null==m_grOffscreen) {
			m_caGrid[nXPos][nYPos]=new CharAttrib(ch, fg, bg);
		} else {
			m_rgchTemp[0]=ch;
			if (null==bg) {
				bg=m_cDefaultBg;
			}
			if (null==fg) {
				fg=m_cDefaultFg;
			}
			int nXPixel=nXPos*m_dCell.width;
			int nYPixel=nYPos*m_dCell.height;
			m_grOffscreen.setColor(bg);
			m_grOffscreen.fillRect(nXPixel, nYPixel, m_dCell.width, m_dCell.height);
			m_grOffscreen.setColor(fg);
			m_grOffscreen.drawChars(m_rgchTemp, 0, 1, nXPixel+m_dOffs.width, nYPixel+m_dOffs.height);
		}
		repaint();
	}
	
	//------------------------------------------------------------------
	// accessor
	public boolean getWrap() {
		return m_bWrap;
	}
	
	//------------------------------------------------------------------
	// accessor
	public void setWrap(boolean bWrap) {
		m_bWrap=bWrap;
	}

	//------------------------------------------------------------------
	// accessor
	public Dimension getGridSize() {
		return new Dimension(m_dGrid);
	}

	//------------------------------------------------------------------
	// Print a string
	public void print(int nXPos, int nYPos, String szText) {
		print(nXPos, nYPos, szText, m_cDefaultFg, m_cDefaultBg);
	}

	//------------------------------------------------------------------
	// Print a string
	public void print(int nXPos, int nYPos, String szText, Color fg, Color bg) {
		Assert.isNotNull(szText);
		char rgchText[]=szText.toCharArray();
		int nStrIndex=0;
		int nStrLen=rgchText.length;
		if (0==nStrLen) {
			return;
		}

		if (null==bg) {
			bg=m_cDefaultBg;
		}
		if (null==fg) {
			fg=m_cDefaultFg;
		}
		
		if (null==m_grOffscreen) {
			// slow backup method
			for (int nIndex=0; nIndex<rgchText.length; nIndex++, nXPos++) {
				if (m_dGrid.width<=nXPos) {
					if (true==m_bWrap) {
						nYPos++;
						nXPos=0;
						if (nYPos>=m_dGrid.height) {
							break;
						}
					} else {
						break;
					}
				}
				m_caGrid[nXPos][nYPos]=new CharAttrib(rgchText[nIndex], fg, bg);
			}
		} else {
			// the fastest thing I can think of
			int nCellWidth=m_dCell.width;
			int nCellHeight=m_dCell.height;
			int nXPixel=nXPos*nCellWidth;
			int nYPixel=nYPos*nCellHeight;
			int nXTextPixel=nXPixel+m_dOffs.width;
			int nYTextPixel=nYPixel+m_dOffs.height;

			do {
				int nXRemain=nXPos-m_dGrid.width;
				if (nXRemain<nStrLen) {
					nXRemain=nStrLen;
				}
				m_grOffscreen.setColor(bg);
				m_grOffscreen.fillRect(nXPixel, nYPixel, nCellWidth*nXRemain, nCellHeight);

				m_grOffscreen.setColor(fg);
				nStrLen-=nXRemain;
				for (;nXRemain>0;--nXRemain, ++nStrIndex, nXTextPixel+=nCellWidth) {
					m_grOffscreen.drawChars(rgchText, nStrIndex, 1, nXTextPixel, nYTextPixel);
				}

				if (nStrLen>0 && true==m_bWrap) {
					// move down and left
					nXPos=0;
					nXPixel=0;
					nYPixel+=nCellHeight;
					nXTextPixel=m_dOffs.width;
					nYTextPixel+=nCellHeight;
					continue;
				}
				break;
			} while(true);
		}


		// Finally, update the screen
		repaint();
	}


	//------------------------------------------------------------------
	// clear a line
	public void clearLine(int nYPos) {
		if (null==m_grOffscreen) {
			for (int nIndex=m_dGrid.width-1; nIndex>=0; nIndex--) {
				m_caGrid[nIndex][nYPos]=null;
			}
		} else {
			m_grOffscreen.setColor(m_cDefaultBg);
			m_grOffscreen.fillRect(0, nYPos*m_dCell.height, m_dCanv.width, m_dCell.height);
		}
		repaint();
	}

	//------------------------------------------------------------------
	// Scroll up
	public void scrollUp(int nLines) {
		Assert.isTrue(nLines>0);
		Assert.isTrue(nLines<m_dGrid.height-1);
		if (null==m_grOffscreen) {
			Assert.error("NYI");
		} else {
			m_grOffscreen.copyArea(0, nLines*m_dCell.height, 
				m_dCanv.width, m_dCanv.height-nLines*m_dCell.height,
				0, -nLines*m_dCell.height);
			m_grOffscreen.setColor(m_cDefaultBg);
			m_grOffscreen.fillRect(0, (m_dGrid.height-nLines)*m_dCell.height, m_dCanv.width, m_dCell.height*nLines);
		}
	}

	//------------------------------------------------------------------
	// Scroll down
	public void scrollDown(int nLines) {
		Assert.isTrue(nLines>0);
		Assert.isTrue(nLines<m_dGrid.height-1);
		if (null==m_grOffscreen) {
			Assert.error("NYI");
		} else {
			m_grOffscreen.copyArea(0, 0, 
				m_dCanv.width, m_dCanv.height-nLines*m_dCell.height,
				0, nLines*m_dCell.height);
			m_grOffscreen.setColor(m_cDefaultBg);
			m_grOffscreen.fillRect(0, 0, m_dCanv.width, m_dCell.height*nLines);
		}
	}

	//------------------------------------------------------------------
	// Scroll left
	public void scrollLeft(int nLines) {
		Assert.isTrue(nLines>0);
		Assert.isTrue(nLines<m_dGrid.width-1);
		if (null==m_grOffscreen) {
			Assert.error("NYI");
		} else {
			m_grOffscreen.copyArea(nLines*m_dCell.width, 0,
				m_dCanv.width-nLines*m_dCell.width, m_dCanv.height,
				-nLines*m_dCell.width, 0);
			m_grOffscreen.setColor(m_cDefaultBg);
			m_grOffscreen.fillRect((m_dGrid.width-nLines)*m_dCell.width, 0, m_dCell.width*nLines, m_dCanv.height);
		}
	}

	//------------------------------------------------------------------
	// Scroll right
	public void scrollRight(int nLines) {
		Assert.isTrue(nLines>0);
		Assert.isTrue(nLines<m_dGrid.width-1);
		if (null==m_grOffscreen) {
			Assert.error("NYI");
		} else {
			m_grOffscreen.copyArea(0, 0, 
				m_dCanv.width-nLines*m_dCell.width, m_dCanv.height,
				nLines*m_dCell.width, 0);
			m_grOffscreen.setColor(m_cDefaultBg);
			m_grOffscreen.fillRect(0, 0, m_dCell.width*nLines, m_dCanv.height);
		}
	}

}

//----------------------------------------------------------------------
// Helper class, to hold a character attribute
class CharAttrib {
	
	//------------------------------------------------------------------
	//member variables
	char ch;
	Color fg;
	Color bg;

	//------------------------------------------------------------------
	// constructor
	CharAttrib(char ch, Color fg, Color bg) {
		this.ch=ch;
		this.fg=fg;
		this.bg=bg;
	}
}
