//
//  JavaDump 1.0
//  Copyright (c) 1997 Matt T. Yourst. All rights reserved.
//
//  File:     JavaDump.java
//  Module:   JavaDump (application)
//  Version:  1.00.053
//  Date:     May - June 1997
//  Author:   Matt Yourst [yourst@laserstars.com]
//  Language: Java 1.1+
//
//  ABSTRACT
//
//  Generates a complete report on the structures within
//  Java class files. Sends report (in HTML format) to
//  standard output; can be redirected to file if desired.
//  Very useful for debugging and exploring class files.
//
package lti.java.javadump;

import java.io.*;
import java.util.*;
import lti.java.jcf.*;

public class javadump
{
  public static void main(String args[]) throws Exception
  {
    if (args.length < 1)
    {
      System.out.println("JavaDump 1.0  Copyright 1997 Matt Yourst");
      System.out.println();
      System.out.println("Purpose:");
      System.out.println("  Generates a complete report on the structures within");
      System.out.println("  Java class files. Sends report (in HTML format) to");
      System.out.println("  standard output; can be redirected to file if desired.");
      System.out.println("Syntax is:");
      System.out.println("  javadump <class file name> [-noconstpool]");
      System.out.println("  -noconstpool = Omit constant pool from report");
      return;
    }
    //
    // Open the class file.
    //
    FileInputStream fist = null;
    try {
      fist = new FileInputStream(args[0]);
    } catch (IOException ioe) {
      System.err.println("JavaDump Error: Cannot open " + args[0]);
      return;
    }
    JcfClassInputStream ist = new JcfClassInputStream(
      new BufferedInputStream(fist, Math.min(fist.available(), 32767)));
    //
    // Read into JcfClassFile object
    //
    JcfClassFile jcf = null;
    try {
      // Read from class input stream, and create structures.
      jcf = new JcfClassFile(ist);
      fist.close();
    } catch (Exception ex) {
      System.err.println("JavaDump Error: Exception while reading class file");
      fist.close();
      throw ex; // Display a stack trace and terminate.
    }
    //
    // Perform complete dump
    //
    int flags = 0;
    if ((args.length >= 2) && (args[1].equals("-noconstpool")))
      flags |= DumpClassToHtml.OMIT_CONSTPOOL;
    new DumpClassToHtml(jcf, System.out, flags).dump();
    //
    // Finally exit
    //
  }
}
