//
//  StripDebug 1.0
//  Copyright (c) 1997 Matt T. Yourst. All rights reserved.
//
//  File:     stripdebug.java
//  Module:   stripdebug (application)
//  Version:  1.00.032
//  Date:     April/May 1997
//  Author:   Matt Yourst [yourst@laserstars.com]
//  Language: Java 1.1+
//
//  ABSTRACT
//
//  Stripdebug removes certain attributes from Java class files which are
//  used solely for debugging. The LineNumberTable, LocalVariables, and
//  SourceFile attributes are removed.
//

package lti.java.stripdebug;

import java.io.*;
import java.util.*;
import lti.java.jcf.*;

public class stripdebug
{
  public static void main(String args[]) throws Exception
  {
    System.out.println("StripDebug 1.0  Copyright 1997 Matt Yourst");
    System.out.println("");
    if (args.length < 1)
    {
      System.out.println("Syntax is: stripdebug [base directory]");
      return;
    }
    String dirNames[] = new File(args[0]).list();
    int totfiles = dirNames.length;
    int totbytes = 0;
    int totBytesOut = 0;
    System.out.println("Stripping debug information from " + args[0] + ", " +
      totfiles + " classes");
    for (int i = 0; i < dirNames.length; i++)
    {
      if (dirNames[i].endsWith(".class"))
      {
        String fullName = args[0] + File.separator + dirNames[i];
        try {
          FileInputStream fist = new FileInputStream(fullName);
          int fistsize = fist.available();
          totbytes += fistsize;
          JcfClassInputStream ist = new JcfClassInputStream(
            new BufferedInputStream(fist, Math.min(fistsize, 32767)));
          // Read class file
          StripDebugClassFile cf = new StripDebugClassFile(ist);
          fist.close();
          // Write class file
          FileOutputStream fost = new FileOutputStream(fullName);
          JcfClassOutputStream ost = new JcfClassOutputStream(
            new BufferedOutputStream(fost, Math.min(fistsize, 32767)));
          cf.write(ost);
          ost.flush();
          totBytesOut += ost.size();
          fost.close();
          System.out.print(".");
        } catch (IOException e) {
          System.err.println("Warning: Cannot read class file " +
            fullName);
        }
      }
    }
    System.out.println("");
    System.out.println("Finished! " + totbytes + " bytes reduced to " +
      totBytesOut + " bytes");
    System.out.println("");
  }  
}
